# coding:utf-8

# @Time: 2020/4/20 15:59
# @Auther: liyubin


import os
import sys
import time
import uuid
import json
import shutil
import socket
import platform
import requests
import configparser
from pathlib import Path


"""
自动注册和生成对应手机的 android/ios work
test_type: api/android/ios/web   对应：0/1/2/3
"""

def get_pc_ip():
    """
    :return: ip34name
    """
    #获取本机电脑名
    myname = socket.gethostname()
    #获取本机ip
    myaddr = socket.gethostbyname(myname)
    ip = myaddr.split(".") if myaddr else ''
    ip34name = ip[2]+ip[3] if isinstance(ip, list) else 'get ip error'
    return ip34name


def get_initialization(test_type, platform_url, workspace_name):
    """
    注册到平台
    :param test_type:
    :param platform_url:
    :param workspace_name:
    :return:
    """
    # 对输入的URL做判断，末尾没加"/" 统一处理加上"/"
    if not platform_url.endswith('/'):
        platform_url += '/'
    # ATP平台端初始化执行端的API接口
    initialization_api_url = platform_url + 'software/execution/initialization/'
    # 如果执行工作区名称为默认值，生成随机数
    if workspace_name == '':
        # 根据当前网卡和时间组成随机数
        workspace_name = uuid.uuid4().hex
    # api、android、ios、web执行端的通用参数
    up_data = {'name': workspace_name, 'execution_type': test_type}
    # api执行端的特殊参数
    if test_type == 0:
        up_data['information'] = json.dumps({'system': platform.system()})
    # android
    elif test_type == 1:
        up_data['information'] = json.dumps({'system': platform.system()})
    # ios
    elif test_type == 2:
        up_data['information'] = json.dumps({'system': platform.system()})
    # 向ATP平台端发送初始化执行端请求
    requests_data = requests.post(initialization_api_url, data=up_data)
    requests_json = requests_data.json()
    print('[Cli info]> ' + str(requests_json))
    # 判断平台端初始化结果
    if requests_json['code'] != 200:
        print('[Cli Error]> 执行端服务初始化失败')
        sys.exit()
    # 附带上经过上面处理的ATP平台URl
    requests_json['data']['platform_url'] = platform_url
    requests_json['data']['redis_ip'] = platform_url.replace('http://','').replace('/','')
    requests_json['data']['test_type'] = test_type
    return requests_json


def create_profile(data, work_space, current_dir, copy_file_list):
    """
    在程序执行的当前目录下创建配置文件和执行程序
    :param data: config.ini 配置信息
    :param work_space: 新创建的work
    :param current_dir: 源执行文件
    :param copy_file_list: 需要拷贝的文件list
    """
    # 处理GIT仓库URL里的特殊字符
    data['git_url_1'] = [git_r.replace('%40', '@') for git_r in data['git_url'] if 'x-atp-sweetest.git' not in git_r][0]
    data['git_url'] = [git_r.replace('%40', '@') for git_r in data['git_url'] if 'x-atp-sweetest.git' in git_r][0]
    config = configparser.ConfigParser()
    # 创建config配置文件
    file = 'config.ini'
    config.read(work_space / file)
    config.add_section('ATP-Server')
    # 平台URL、GIT仓库URL、工作空间名、队列IP、队列KEY、队列端口号、注册ID
    config.set('ATP-Server', 'platform_url', json.dumps(list(map(ord, data['platform_url']))))
    config.set('ATP-Server', 'name', data['name'])
    config.set('ATP-Server', 'git_url', json.dumps(list(map(ord, data['git_url']))))
    config.set('ATP-Server', 'git_url_1', json.dumps(list(map(ord, data['git_url_1']))))
    config.set('ATP-Server', 'redis_ip', json.dumps(list(map(ord, data['redis_ip']))))
    config.set('ATP-Server', 'redis_key', json.dumps(list(map(ord, str(data['redis_key'])))))
    config.set('ATP-Server', 'redis_port', json.dumps(list(map(ord, str(data['redis_port'])))))
    config.set('ATP-Server', 'execution_id', str(data['execution_id']))
    config.set('ATP-Server', 'iostagen_file', json.dumps(list(map(ord, str(data['iostagen_file'])))))
    config.set('ATP-Server', 'test_type', str(data['test_type']))
    # 保存config配置文件
    with open(work_space / file, 'w', encoding='utf-8') as configfile:
        config.write(configfile)
    # 拷贝Android_IOS_Work执行文件到工作空间

    for py in copy_file_list:
        shutil.copy(current_dir / py, work_space / py)

def new_workspace(sys_name, data, copy_file_list, iostagen_file, diy_redis_ip):
    """
    创建新的work
    :param sys_name:
    :param data:
    :param copy_file_list:
    :param iostagen_file:
    :return:
    """
    data_dict = {
        'name': data['name'],
        'platform_url': data['platform_url'],
        'git_url': data['git_url'],
        'redis_ip': data['redis_ip'] if not diy_redis_ip else diy_redis_ip,
        'redis_key': data['redis_key'],
        'redis_port': data['redis_port'],
        'execution_id': data['execution_id'],
        'iostagen_file': iostagen_file,
        'test_type': data['test_type'],
    }
    current_path = os.path.abspath(__file__)
    # 需要拷贝的x-app-package下的文件
    current_dir = Path(os.path.join(os.path.dirname(current_path), 'copy_file'))
    cwd_dir = Path.cwd()
    work_space = cwd_dir / ('Android_IOS_Work_%s' % data_dict['name'])
    os.mkdir(str(work_space))
    create_profile(data=data_dict, work_space=work_space, current_dir=current_dir, copy_file_list=copy_file_list)

    # 新app服务路径
    # new_server_path = Path(os.path.join(os.path.dirname(current_path), 'Android_IOS_Work_' + data_dict['name']))
    new_server_path = 'Android_IOS_Work_' + data_dict['name'] # client放在内部调用后，不展示super-sweetest的安装路径
    # print('\n###   工作空间   ###\n%s\n' % new_server_path)

    # 后台执行atp执行端
    if sys_name == 'Linux':
        # daemon_start(data=data_dict, work_space=work_space)
        print('###   X-ATP自动化测试执行端服务已在后台运行   ###')
    elif sys_name == 'Windows':
        print('###   您需要手动启动Android自动化work 服务   ###\n1、进入目录：%s\n2、双击运行程序： %s\n' % (new_server_path, 'run_this.py'))
    else:
        print('###   您需要手动启动ios自动化work 服务   ###\n1、进入目录：%s\n2、运行程序： %s\n' % (new_server_path, 'run_this.py'))


def create_android_ios_work(mobile_list, test_type, platform_url, copy_file_list, iostagen_file, diy_redis_ip=None):
    """
    创建当前接入手机的Android/ios自动化work
    :param mobile_list:
    :param test_type:
    :param platform_url:
    :param copy_file_list:
    :param iostagen_file:
    :param diy_redis_ip: 自定义redis地址
    :return:
    """

    localtime_ = time.asctime(time.localtime(time.time()))
    work_list_file = './work_name_list.json'

    for device in mobile_list:
        # 获取已连接的android手机名称
        device_ = device.get('device_name')
        if 'unauthorized' in device_:
            break
        mobile_name = device.get('mobile_name') if int(test_type) == 1 else device.get('version')

        # mobilename_devicesname
        ip34 = get_pc_ip()
        if ip34 == '01':
            break
        work_name = mobile_name.replace(' ', '') + '-' + ip34 + '_' + device_.replace(':', '##')

        # 首次创建
        if not os.path.exists(work_list_file):
            with open(work_list_file, 'w+')as fp:
                json.dump(['work_name'], fp)


        # 读取work_list_file 创建历史
        with open(work_list_file, 'rb')as fp1:
            work_name_list = json.load(fp1)

        # 已存在历史中
        if work_name in work_name_list:
            # 是否存在 Android自动化work 目录
            if not os.path.exists('./Android_IOS_Work_%s' % work_name):
                print(localtime_ + ' 本机创建过，但不存在  对应手机: %s 的Android/ios自动化 work 目录...   \n正在创建...' % work_name)
                # 注册到平台
                data_json = get_initialization(test_type, platform_url, work_name)
                # 创建新的执行程序
                new_workspace(sys_name=platform.system(), data=data_json['data'], copy_file_list=copy_file_list, iostagen_file=iostagen_file, diy_redis_ip=diy_redis_ip)
                # 已在work_list历史不添加

        else:
            # 清理历史清单信息异常时，可能存在的 Android/ios自动化work 目录
            if os.path.exists('./Android_IOS_Work_%s' % work_name):
                print(localtime_ + ' 清理已存在: %s  的Android/ios自动化work 目录...' % work_name)
                shutil.rmtree('./Android_IOS_Work_%s' % work_name)

            print(localtime_ + ' 不存在 对应手机: %s  的Android/ios自动化work 目录...   \n正在创建...' % work_name)
            # 注册到平台
            data_json = get_initialization(test_type, platform_url, work_name)
            # 创建新的执行程序
            new_workspace(sys_name=platform.system(), data=data_json['data'], copy_file_list=copy_file_list, iostagen_file=iostagen_file, diy_redis_ip=diy_redis_ip)

            # 将此设备加入已创建历史
            with open(work_list_file, 'w+')as fp2:
                work_name_list.append(work_name)
                json.dump(work_name_list, fp2)


if __name__ == '__main__':

    DEVICE_LIST = ['1197cd5e']  # 建议写手机名和下划线devicename
    TEST_TYPE = 1  # android
    PLATFORM_URL = 'http://127.0.0.1'  # 平台地址
    COPY_FILE_LIST = ['test_case.py', 'run_this.py']  # copy_file下需要拷贝的文件

    while True:
        time.sleep(5)
        create_android_ios_work(DEVICE_LIST, TEST_TYPE, PLATFORM_URL, COPY_FILE_LIST,'iostagen_file')



from pathlib import Path
from PIL import Image
from PIL import ImageChops
import math
import operator
import time
from functools import reduce
from super_sweetest.globals import g, now
from super_sweetest.log import logger
from super_sweetest.utility import mkdir
from airtest.core.api import snapshot,sleep



def crop(element, src, target):
    location = element.location
    size = element.size
    im = Image.open(src)
    left = location['x']
    top = location['y']
    right = location['x'] + size['width']
    bottom = location['y'] + size['height']
    im = im.crop((left, top, right, bottom))
    im.save(target)


def blank(src, boxs):
    white = Image.new('RGB', (5000, 5000), 'white')
    im = Image.open(src)
    for box in boxs:
        w = white.crop(box)
        im.paste(w, box[:2])
    im.save(src)


def cut(src, target, box):
    im = Image.open(src)
    im = im.crop(box)
    im.save(target)


def get_screenshot(file_path):
    if g.headless:
        width = g.driver.execute_script(
            "return Math.max(document.body.scrollWidth, document.documentElement.clientWidth, document.documentElement.scrollWidth, document.documentElement.offsetWidth);")
        height = g.driver.execute_script(
            "return Math.max(document.body.scrollHeight, document.documentElement.clientHeight, document.documentElement.scrollHeight, document.documentElement.offsetHeight);")
        g.driver.set_window_size(width, height)
        time.sleep(3)
    g.driver.get_screenshot_as_file(file_path)
    if g.headless:
        g.driver.set_window_size(1920, 1080)


def get_air_screenshot(file_path):
    sleep(1.5)
    snapshot(file_path)


def check_for_pil(image1, image2):
    histogram1 = image1.histogram()
    histogram2 = image2.histogram()
    differ = math.sqrt(reduce(operator.add, list(
        map(lambda a, b: (a - b) ** 2, histogram1, histogram2))) / len(histogram1))
    diff = ImageChops.difference(image1.convert('RGB'), image2.convert('RGB'))
    return diff, differ


class Snapshot:
    def __init__(self):
        snapshot_plan = Path('snapshot') / g.plan_name
        self.snapshot_folder = snapshot_plan / g.start_time[1:]
        snapshot_expected = Path('snapshot') / 'expected'
        self.expected_folder = snapshot_expected / g.plan_name
        for p in (snapshot_plan, self.snapshot_folder, snapshot_expected, self.expected_folder):
            mkdir(p)

    def pre(self, step, label):
        self.label = label
        self.screen_flag = False
        self.element_flag = False

        # 处理输出数据中的截图设置
        self.output = {}
        for k, v in dict(step['output'].items()).items():
            if v == '#ScreenShot':
                self.output['#ScreenShot'] = k
                step['output'].pop(k)
                self.screen_flag = True
            if v == '#ElementShot':
                self.output['#ElementShot'] = k
                step['output'].pop(k)
                self.element_flag = True

        # 处理图片比较
        self.expected = {}
        for data in (step['data'], step['expected']):
            if '#ScreenShot' in data:
                self.screen_flag = True
                p = Path(data['#ScreenShot']).stem
                self.expected['#ScreenName'] = '(' + p.split('[')[1].split(']')[0] + ')' if '[' in p else p
                if Path(data['#ScreenShot']).is_file():

                    step['snapshot']['Expected_Screen'] = data.pop('#ScreenShot')
                else:
                    step['snapshot']['Expected_Screen'] = str(self.expected_folder / data.pop('#ScreenShot'))

            if '#ElementShot' in data:
                self.element_flag = True
                p = Path(data['#ElementShot']).stem
                self.expected['#ElementName'] = '(' + p.split('[')[1].split(']')[0] + ')' if '[' in p else p
                if Path(data['#ElementShot']).is_file():
                    step['snapshot']['Expected_Element'] = data.pop('#ElementShot')
                else:
                    step['snapshot']['Expected_Element'] = str(self.expected_folder / data.pop('#ElementShot'))

            if '#cut' in data:
                self.expected['#cut'] = data.pop('#cut')
            if '#blank' in data:
                self.expected['#blank'] = data.pop('#blank')


    def web_screen(self, step, element):
        # 截图
        self.web_appium_screen(step, element)

    def web_check(self, step, element):
        # 比对
        self.web_appium_check(step, element)

    def web_shot(self, step, element):
        self.web_screen(step, element)
        self.web_check(step, element)


    # ################################################
    # appium
    def android_screen(self, step, element):
        # 截图
        self.web_appium_screen(step, element)

    def android_check(self, step, element):
        # 比对
        self.web_appium_check(step, element)

    def android_shot(self, step, element):
        self.android_screen(step, element)
        self.android_check(step, element)


    #########################################
    # air_android shot
    def air_android_screen(self, step):
        # 截图
        screen_v = self.output.get('#ScreenShot', '')

        if g.snapshot or self.screen_flag or self.element_flag:
            if self.expected.get('#ScreenName'):
                screen_name = self.expected['#ScreenName']
            elif screen_v:
                screen_name = screen_v
            else:
                screen_name = ''
            if screen_name:
                file_name = self.label + now() + '#Screen' + '[' + screen_name + ']' + '.png'
            else:
                file_name = self.label + now() + '#Screen' + '.png'
            step['snapshot']['Real_Screen'] = str(
                Path(self.snapshot_folder) / file_name)
            get_air_screenshot(step['snapshot']['Real_Screen'])
            if screen_v:
                g.var[screen_v] = step['snapshot']['Real_Screen']


    def air_android_check(self, step):

        def deep(src):
            # 把不需要比较的部分贴白
            if self.expected.get('#blank'):
                blank(src, eval(self.expected.get('#blank')))
            # 裁剪需要比较的部分
            if self.expected.get('#cut'):
                cut(src, src, eval(self.expected.get('#cut')))

        if Path(step['snapshot'].get('Expected_Screen', '')).is_file():
            # 屏幕截图比较
            image1 = Image.open(step['snapshot']['Expected_Screen'])
            image2 = Image.open(step['snapshot']['Real_Screen'])
            deep(step['snapshot']['Real_Screen'])
            diff, differ = check_for_pil(image1,image2)
            if differ == 0.0:
                # 图片间没有任何不同
                logger.info('SnapShot: ScreenShot is the same')
            else:
                file_name = self.label + now() + 'Diff_Screen' + '.png'
                step['snapshot']['Diff_Screen'] = str(
                    Path(self.snapshot_folder) / file_name)
                diff.save(step['snapshot']['Diff_Screen'])
                raise Exception('SnapShot: ScreenShot is diff: %s' % differ)
        elif step['snapshot'].get('Expected_Screen'):
            get_air_screenshot(step['snapshot']['Expected_Screen'])
            deep(step['snapshot']['Expected_Screen'])


    def air_android_shot(self, step):
        self.air_android_screen(step)
        self.air_android_check(step)


    # windows
    def windwos_capture(self, dialog, step):
        # 截图
        screen_v = self.output.get('#ScreenShot', '')
        element_v = self.output.get('#ElementShot', '')

        if g.snapshot or self.screen_flag:
            if self.expected.get('#ScreenName'):
                screen_name = self.expected['#ScreenName']
            elif screen_v:
                screen_name = screen_v
            else:
                screen_name = ''
            if screen_name:
                file_name = self.label + now() + '#Screen' + '[' + screen_name + ']' + '.png'
            else:
                file_name = self.label + now() + '#Screen' + '.png'
            step['snapshot']['Real_Screen'] = str(Path(self.snapshot_folder) / file_name)
            pic = dialog.capture_as_image()
            pic.save(step['snapshot']['Real_Screen'])
            if screen_v:
                g.var[screen_v] = step['snapshot']['Real_Screen']

        if element_v:
            file_name = self.label + now() + '#Element' + '[' + element_v + ']' + '.png'
            step['snapshot']['Real_Element'] = str(Path(self.snapshot_folder) / file_name)

            element = step['element']
            if dialog.backend.name == 'win32':
                pic = dialog.window(best_match=element).capture_as_image()
            elif dialog.backend.name == 'uia':
                pic = dialog.child_window(best_match=element).capture_as_image()
            pic.save(step['snapshot']['Real_Screen'])
            g.var[element_v] = step['snapshot']['Real_Element']


    def windwos_check(self, dialog, step):
        element = step['element']
        if Path(step['snapshot'].get('Expected_Screen', '')).is_file():
            # 屏幕截图比较
            image1 = Image.open(step['snapshot']['Expected_Screen'])
            image2 = Image.open(step['snapshot']['Real_Screen'])
            diff, differ = check_for_pil(image1, image2)
            if differ == 0.0:
                # 图片间没有任何不同
                logger.info('SnapShot: ScreenShot is the same')
            else:
                file_name = self.label + now() + 'Diff_Screen' + '.png'
                step['snapshot']['Diff_Screen'] = str(
                    Path(self.snapshot_folder) / file_name)
                diff.save(step['snapshot']['Diff_Screen'])
                raise Exception('SnapShot: ScreenShot is diff: %s' % differ)
        elif step['snapshot'].get('Expected_Screen'):
            pic = dialog.capture_as_image()
            pic.save(step['snapshot']['Expected_Screen'])

        if Path(step['snapshot'].get('Expected_Element', '')).is_file():
            file_name = self.label + now() + '#Element' + '.png'
            step['snapshot']['Real_Element'] = str(Path(self.snapshot_folder) / file_name)
            if dialog.backend.name == 'win32':
                pic = dialog.window(best_match=element).capture_as_image()
            elif dialog.backend.name == 'uia':
                pic = dialog.child_window(best_match=element).capture_as_image()
            pic.save(step['snapshot']['Real_Element'])

            # 屏幕截图比较
            image1 = Image.open(step['snapshot']['Expected_Element'])
            image2 = Image.open(step['snapshot']['Real_Element'])
            diff, differ = check_for_pil(image1, image2)
            if differ == 0.0:
                logger.info('SnapShot: ElementShot is the same')
            else:
                file_name = self.label + now() + 'Diff_Element' + '.png'
                step['snapshot']['Diff_Element'] = str(
                    Path(self.snapshot_folder) / file_name)
                diff.save(step['snapshot']['Diff_Element'])
                raise Exception('SnapShot: ElementShot is diff: %s' % differ)
        elif step['snapshot'].get('Expected_Element'):
            if dialog.backend.name == 'win32':
                pic = dialog.window(best_match=element).capture_as_image()
            elif dialog.backend.name == 'uia':
                pic = dialog.child_window(best_match=element).capture_as_image()
            pic.save(step['snapshot']['Expected_Element'])

    def windows_shot(self, dialog, step):
        self.windwos_capture(dialog, step)
        self.windwos_check(dialog, step)



    # 公用 截图比对
    def web_appium_screen(self, step, element):
        # 截图 selenium/appium
        screen_v = self.output.get('#ScreenShot', '')
        element_v = self.output.get('#ElementShot', '')

        if g.snapshot or self.screen_flag or self.element_flag:
            from selenium.webdriver.support import expected_conditions as EC
            if not EC.alert_is_present()(g.driver):
                if self.expected.get('#ScreenName'):
                    screen_name = self.expected['#ScreenName']
                elif screen_v:
                    screen_name = screen_v
                else:
                    screen_name = ''
                if screen_name:
                    file_name = self.label + now() + '#Screen' + '[' + screen_name + ']' + '.png'
                else:
                    file_name = self.label + now() + '#Screen' + '.png'
                step['snapshot']['Real_Screen'] = str(
                    Path(self.snapshot_folder) / file_name)
                get_screenshot(step['snapshot']['Real_Screen'])
                if screen_v:
                    g.var[screen_v] = step['snapshot']['Real_Screen']

        if element_v:
            file_name = self.label + now() + '#Element' + '[' + element_v + ']' + '.png'
            step['snapshot']['Real_Element'] = str(Path(self.snapshot_folder) / file_name)
            crop(element, step['snapshot']['Real_Screen'], step['snapshot']['Real_Element'])
            g.var[element_v] = step['snapshot']['Real_Element']


    def web_appium_check(self, step, element):
        def deep(src):
            # 把不需要比较的部分贴白
            if self.expected.get('#blank'):
                blank(src, eval(self.expected.get('#blank')))
            # 裁剪需要比较的部分
            if self.expected.get('#cut'):
                cut(src, src, eval(self.expected.get('#cut')))

        if Path(step['snapshot'].get('Expected_Screen', '')).is_file():
            # 屏幕截图比较
            image1 = Image.open(step['snapshot']['Expected_Screen'])
            image2 = Image.open(step['snapshot']['Real_Screen'])
            deep(step['snapshot']['Real_Screen'])
            diff, differ = check_for_pil(image1, image2)
            if differ == 0.0:
                # 图片间没有任何不同
                logger.info('SnapShot: ScreenShot is the same')
            else:
                file_name = self.label + now() + 'Diff_Screen' + '.png'
                step['snapshot']['Diff_Screen'] = str(
                    Path(self.snapshot_folder) / file_name)
                diff.save(step['snapshot']['Diff_Screen'])
                raise Exception('SnapShot: ScreenShot is diff: %s' % differ)
        elif step['snapshot'].get('Expected_Screen'):
            get_screenshot(step['snapshot']['Expected_Screen'])
            deep(step['snapshot']['Expected_Screen'])

        if Path(step['snapshot'].get('Expected_Element', '')).is_file():
            file_name = self.label + now() + '#Element' + '[' + self.expected['#ElementName'] + ']' + '.png'
            step['snapshot']['Real_Element'] = str(Path(self.snapshot_folder) / file_name)
            crop(element, step['snapshot']['Real_Screen'], step['snapshot']['Real_Element'])
            deep(step['snapshot']['Real_Element'])

            # 屏幕截图比较
            image1 = Image.open(step['snapshot']['Expected_Element'])
            image2 = Image.open(step['snapshot']['Real_Element'])
            diff, differ = check_for_pil(image1, image2)
            if differ == 0.0:
                logger.info('SnapShot: ElementShot is the same')
            else:
                file_name = self.label + now() + 'Diff_Element' + '.png'
                step['snapshot']['Diff_Element'] = str(
                    Path(self.snapshot_folder) / file_name)
                diff.save(step['snapshot']['Diff_Element'])
                raise Exception('SnapShot: ElementShot is diff: %s' % differ)
        elif step['snapshot'].get('Expected_Element'):
            crop(element, step['snapshot']['Real_Screen'], step['snapshot']['Expected_Element'])
            deep(step['snapshot']['Expected_Element'])


from copy import deepcopy
from super_sweetest.globals import g
from super_sweetest.elements import e
from super_sweetest.log import logger
from super_sweetest.parse import data_format
from super_sweetest.database import DB
from super_sweetest.utility import replace_dict, compare


def execute(step):
    # 先处理循环结束条件
    condition = ''
    for k in ('循环结束条件', 'condition'):
        if step['data'].get(k):
            condition = step['data'].get(k)
            del step['data'][k]
    if condition.lower() in ('成功', 'success'):
        condition = 'success'
    elif condition.lower() in ('失败', 'failure'):
        condition = 'failure'

    # 执行赋值操作
    data = step['data']
    for k, v in data.items():
        g.var[k] = v

    from super_sweetest.testcase import TestCase
    element = step['element']
    times = 1
    _element = element.split('*')

    # snippet 执行失败是否退出标志
    flag = True
    if element[-1] == '*':
        flag = False

    if len(_element) >= 2:
        element = _element[0]
        if _element[1].upper() == 'N':
            times = 999
        else:
            times = int(_element[1])

    # 初始化测试片段执行结果
    result = 'success'
    steps = []
    if element != '变量赋值':
        g.var['_last_'] = False
        for t in range(times):
            if t > 0:
                _data = data_format(str(step['_data']))
                replace_dict(_data)
                for k, v in _data.items():
                    g.var[k] = v
            testcase = deepcopy(g.snippet[element])
            tc = TestCase(testcase)
            tc.run()
            for s in testcase['steps']:
                s['no'] = str(step['no']) + '*' + \
                    str(t + 1) + '-' + str(s['no'])
            steps += testcase['steps']
            # 用例片段执行失败时
            if testcase['result'] != 'success':
                result = testcase['result']
                # 循环退出条件为失败，则直接返回，返回结果是 success
                if condition == 'failure':
                    return 'success', testcase['steps']
                # 如果没有结束条件，且直接退出标志位为真，则返回结果
                if not condition and flag:
                    return result, steps

            # 用例片段执行成功时
            else:
                # 如果循环退出条件是成功，则直接返回，返回结果是 success
                if condition == 'success':
                    return 'success', testcase['steps']
            
            if g.var['_last_']:
                g.var['_last_'] = False
                break
        # 执行结束，还没有触发循环退出条件，则返回结果为 failure
        if condition:
            return 'failure', testcase['steps']
    return result, steps


def dedup(text):
    '''
    去掉 text 中括号及其包含的字符
    '''
    _text = ''
    n = 0

    for s in text:
        if s not in ( '(', ')'):
            if n <= 0:
                _text += s
        elif s == '(':
            n += 1
        elif s == ')':
            n -= 1
    return _text    


def sql(step):
    element = step['element']
    _sql = e.get(element)[1]

    logger.info('SQL: %s' % repr(_sql))
    # 获取连接参数
    value = e.get(step['page'] + '-' + 'config')[1]
    arg = data_format(value)

    if step['page'] not in g.db.keys():
        g.db[step['page']] = DB(arg)
    if _sql.lower().startswith('select'):
        row = g.db[step['page']].fetchone(_sql)
        logger.info('SQL result: %s' % repr(row))
        if not row:
            raise Exception('*** Fetch None ***')
    else:
        g.db[step['page']].execute(_sql)

    result = {}
    if _sql.lower().startswith('select'):
        text = _sql[6:].split('FROM')[0].split('from')[0].strip()
        keys = dedup(text).split(',')
        for i, k in enumerate(keys):
            keys[i] = k.split(' ')[-1]
        result = dict(zip(keys, row))
        logger.info('keys result: %s' % repr(result))

    data = step['data']
    if not data:
        data = step['expected']
    if data:
        for key in data:
            sv, pv = data[key], result[key]
            logger.info('key: %s, expect: %s, real: %s' %
                        (repr(key), repr(sv), repr(pv)))

            compare(sv, pv)

    output = step['output']
    if output:
        logger.info('output: %s' % repr(output))
        for key in output:
            g.var[key] = result[output[key]]



# coding:utf-8

# @Time: 2020/12/10 12:16
# @Auther: liyubin

from super_sweetest.client.app.run_sync_device import run_sync_device_


"""
1、 手机信息同步，单独运行保障执行用例过程中同步

2、 检测接入的手机和创建Android/ios自动化work 目录

    部署需关注：配置 TEST_TYPE 、PLATFORM_URL
"""

TEST_TYPE = 1  # 创建 Android 标识 1, iOS 标识 2


run_sync_device_(test_type=TEST_TYPE)




from time import sleep
import re
from super_sweetest.log import logger
from super_sweetest.globals import g, now, timestamp
from super_sweetest.elements import e
from super_sweetest.windows import w
from super_sweetest.snapshot import Snapshot, get_air_screenshot
from super_sweetest.keywords import web, common, mobile, http, files, air_mobile, mqtt
from super_sweetest.config import web_keywords, mobile_keywords, http_keywords, windows_keywords, files_keywords
from super_sweetest.config import air_mobile_keywords, mqtt_keywords
from super_sweetest.utility import replace_dict, replace
from super_sweetest.servers.photo_handle import base64_photo


def elements_format(page, element):
    if not page:
        page = g.current_page

    if not element:
        return page, '', element

    if page in ('SNIPPET', '用例片段') or element in ('变量赋值',):
        return page, '', element

    elements = element.split('|')
    if len(elements) == 1:
        custom, el = e.have(page, element)
        g.current_page = page
        return page, custom, el
    else:
        els = []
        for _element in elements:
            custom, el = e.have(page, _element.strip())
            els.append(el)
        g.current_page = page
        return page, custom, els


def v_data(d, _d):
    data = ''
    if ',,' in str(_d):
        s = ',,'
    else:
        s = ','

    for k, v in d.items():
        data += k + '=' + str(v) + s

    if s ==',,':
        return data[:-2]
    else:
        return data[:-1]


def test_v_data():
    data = {'a': 1, 'b': 'b'}
    _data = "{'a': 1,, 'b': 'b'}"
    return v_data(data, _data)


class TestCase:
    def __init__(self, testcase):
        self.testcase = testcase
        self.snippet_steps = {}

    def run(self):
        logger.info('>>> Run the TestCase: %s|%s' %
                    (self.testcase['id'], self.testcase['title']))
        self.testcase['result'] = 'success'
        self.testcase['report'] = ''
        if_result = ''

        for index, step in enumerate(self.testcase['steps']):
            # 统计开始时间
            step['start_timestamp'] = timestamp()
            step['snapshot'] = {}

            # if 为否，不执行 then 语句
            if step['control'] == '>' and not if_result:
                step['score'] = '-'
                step['end_timestamp'] = timestamp()
                logger.info('Skipped the <then> Step: %s|%s|%s|%s' %(step['no'], step['page'], step['keyword'], step['element']))
                continue

            # if 为真，不执行 else 语句
            if step['control'] == '<' and if_result:
                step['score'] = '-'
                step['end_timestamp'] = timestamp()
                logger.info('Skipped the <else> Step: %s|%s|%s|%s' %(step['no'], step['page'], step['keyword'], step['element']))
                continue

            if not (g.platform.lower() in ('windows',) and step['keyword'] and step['keyword'].upper() in windows_keywords):
                step['page'], step['custom'], step['element'] = elements_format(
                    step['page'], step['element'])
            label = g.sheet_name + '#' + \
                self.testcase['id'] + '#' + str(step['no']).replace('<', '(').replace('>', ')').replace('*', 'x')

            logger.info('Run the Step: %s|%s|%s|%s' %
                        (step['no'], step['page'], step['keyword'], step['element']))

            snap = Snapshot()
            try:
                after_function = step['data'].pop('AFTER_FUNCTION', '')

                # 处理强制等待时间
                t = step['data'].pop('等待时间', 0)
                sleep(float(t))

                if isinstance(step['element'], str):
                    step['element'] = replace(step['element'])
                    step['_element'] = step['element']
                elif isinstance(step['element'], list):
                    for i in range(len(step['element'])):
                        step['element'][i] = replace(step['element'][i])
                    step['_element'] = '|'.join(step['element'])

                # 变量替换
                replace_dict(step['data'])
                replace_dict(step['expected'])

                step['data'].pop('BEFORE_FUNCTION', '')

                step['vdata'] = v_data(step['data'], step['_data'])

                if g.platform.lower() in ('desktop',) and step['keyword'] and  step['keyword'].upper() in web_keywords:
                    # 处理截图数据
                    snap.pre(step, label)

                    if  step['keyword'] and step['keyword'].upper() not in ('MESSAGE', '对话框'):
                        # 判断页面是否已和窗口做了关联，如果没有，就关联当前窗口，如果已关联，则判断是否需要切换
                        w.switch_window(step['page'])
                        # 切换 frame 处理，支持变量替换
                        frame = replace(step['custom'])
                        w.switch_frame(frame)

                    # 根据关键字调用关键字实现
                    element = getattr(web, step['keyword'].lower())(step)
                    snap.web_shot(step, element)

                elif g.platform.lower() in ('ios', 'android') and  step['keyword'] and step['keyword'].upper() in mobile_keywords:
                    # 处理截图数据
                    snap.pre(step, label)

                    # 切換 context 處理
                    context = replace(step['custom']).strip()
                    w.switch_context(context)

                    if w.current_context.startswith('WEBVIEW'):
                        # 切换标签页
                        tab = step['data'].get('标签页')
                        if tab:
                            del step['data']['标签页']
                            g.driver.switch_to_window(w.windows[tab])
                        logger.info('Current Context: %s' %
                                    repr(w.current_context))

                    # 根据关键字调用关键字实现
                    # getattr(mobile, step['keyword'].lower())(step)
                    # 根据关键字调用关键字实现, 反回对象属性值
                    element = getattr(mobile, step['keyword'].lower())(step)
                    snap.android_shot(step, element)

                elif g.platform.lower() in ('air_android', 'air_ios') and step['keyword'] and step['keyword'].upper() in air_mobile_keywords:
                    # 处理截图数据
                    snap.pre(step, label)

                    # 根据关键字调用关键字实现
                    getattr(air_mobile, step['keyword'].lower())(step)
                    snap.air_android_shot(step)
                    # 每个步骤后AI识别
                    from super_sweetest.keywords.air_mobile import ai_
                    # 获取testcase中的id/title
                    ai_(step, self.testcase)

                elif step.get('keyword') and step.get('keyword').upper() in mqtt_keywords:
                    getattr(mqtt, step.get('keyword').lower())(step)

                elif g.platform.lower() in ('windows',) and  step['keyword'] and step['keyword'].upper() in windows_keywords:
                    from super_sweetest.keywords import windows
                    _page = ''
                    if step['page'].startswith('#'):
                        _page = step['page'][1:]
                        page = [x for x in re.split(r'(<|>)', _page) if x !='']
                    else:
                        page = [x for x in re.split(r'(<|>)', step['page']) if x !='']

                    if _page:
                        dialog = g.windows['#'].dialog(page)
                    else:
                        dialog = g.windows['default'].dialog(page)
                    #dialog.wait('ready')

                    snap.pre(step, label)

                    # 根据关键字调用关键字实现
                    getattr(windows, step['keyword'].lower())(dialog, step)
                    snap.windows_shot(dialog, step)

                elif  step.get('keyword') and step.get('keyword').upper() in http_keywords:
                    # 根据关键字调用关键字实现
                    getattr(http, step['keyword'].lower())(step)

                elif  step['keyword'] and step['keyword'].upper() in files_keywords:
                    # 根据关键字调用关键字实现
                    getattr(files, step['keyword'].lower())(step)

                elif  step['keyword'] and step['keyword'].lower() == 'execute':
                    result, steps = getattr(
                        common, step['keyword'].lower())(step)
                    self.testcase['result'] = result
                    if step['page'] in ('SNIPPET', '用例片段'):
                        self.snippet_steps[index + 1] = steps
                    if result != 'success':
                        step['end_timestamp'] = timestamp()
                        break

                    # elif step['page'] in ('SCRIPT', '脚本'):
                    #     # 判断页面是否已和窗口做了关联，如果没有，就关联当前窗口，如果已关联，则判断是否需要切换
                    #     w.switch_window(step['page'])
                    #     # 切换 frame 处理，支持变量替换
                    #     frame = replace(step['custom'])
                    #     w.switch_frame(frame)
                    #     common.script(step)

                else:
                    # 根据关键字调用关键字实现
                    if step['keyword']:
                        getattr(common, step['keyword'].lower())(step)
                logger.info('--- success ---')
                step['score'] = 'OK'

                # if 语句结果赋值
                if step['control'] == '^':
                    if_result = True

                if after_function:
                    replace_dict({'after_function': after_function})
                # 操作后，等待0.2秒
                sleep(0.2)
            except Exception as exception:

                if g.platform.lower() in ('desktop',) and  step['keyword'] and step['keyword'].upper() in web_keywords:
                    file_name = label + now() + '#Failure' +'.png'
                    step['snapshot']['Failure'] = str(snap.snapshot_folder / file_name)
                    try:
                        if w.frame != 0:
                            g.driver.switch_to.default_content()
                            w.frame = 0
                        g.driver.get_screenshot_as_file(step['snapshot']['Failure'])
                    except:
                        logger.exception(
                            '*** save the screenshot is failure ***')

                elif g.platform.lower() in ('ios', 'android') and  step['keyword'] and step['keyword'].upper() in mobile_keywords:
                    file_name = label + now() + '#Failure' +'.png'
                    step['snapshot']['Failure'] = str(snap.snapshot_folder / file_name)
                    try:
                        g.driver.switch_to.context('NATIVE_APP')
                        w.current_context = 'NATIVE_APP'
                        g.driver.get_screenshot_as_file(u'%s' %step['snapshot']['Failure'])

                        #  失败步骤图片名称转 base64
                        base64Failure = base64_photo(step['snapshot']['Failure'], True)
                        #  snapshot步骤新增key: base64Failure, value: base64_data
                        step['snapshot']['base64Failure'] = base64Failure

                    except:
                        logger.exception('*** save the screenshot is failure ***')

                elif g.platform.lower() in ('air_android', 'air_ios') and step['keyword'] and step['keyword'].upper() in air_mobile_keywords:
                    file_name = label + now() + '#Failure' +'.png'
                    step['snapshot']['Failure'] = str(snap.snapshot_folder / file_name)
                    try:
                        get_air_screenshot(u'%s' %step['snapshot']['Failure'])

                        #  失败步骤图片名称转 base64
                        base64Failure = base64_photo(step['snapshot']['Failure'], True)
                        #  snapshot步骤新增key: base64Failure, value: base64_data
                        step['snapshot']['base64Failure'] = base64Failure

                    except:
                        logger.exception('*** save the screenshot is failure ***')

                logger.exception('+++ failure +++')
                step['score'] = 'NO'

                step['remark'] += str(exception)
                step['end_timestamp'] = timestamp()

                # if 语句结果赋值
                if step['control'] == '^':
                    if_result = False
                    continue

                self.testcase['result'] = 'failure'
                self.testcase['report'] = 'step-%s|%s|%s: %s' % (
                    step['no'], step['keyword'], step['element'], exception)
                break

            # 统计结束时间
            step['end_timestamp'] = timestamp()

        steps = []
        i = 0
        for k in self.snippet_steps:
            steps += self.testcase['steps'][i:k] + self.snippet_steps[k]
            i = k
        steps += self.testcase['steps'][i:]
        self.testcase['steps'] = steps



from pywinauto.keyboard import send_keys as sendkeys
import re
from super_sweetest.log import logger
from super_sweetest.globals import g
from super_sweetest.utility import compare

class Windows():
    def __init__(self, app):
        self.app = app
        self.backend = app.backend.name
        self.dialogs = []

    def dialog(self, page):
        if page == []:
            if self.dialogs:
                return self.dialogs[-1]
            else:           
                raise Exception('Dialog: your page start with "", but there is no parent dialog')
        elif  page[0] == '<':
            if len(self.dialogs) >= 2:
                self.dialogs.pop()
                return self.dialog(page[1:])
            else:
                raise Exception('Dialog: your page start with "<", but the parent is less than 1 dialog')            
        elif page[0] == '>':
            if self.dialogs:
                if self.backend == 'win32':
                    current_dialog = self.app.window(best_match=page[1])
                elif self.backend == 'uia':
                    current_dialog = self.dialogs[-1].child_window(best_match=page[1])
                self.dialogs.append(current_dialog)
                return self.dialog(page[2:])
            else:
                raise Exception('Dialog: your page start with ">", but there is no parent dialog')
        else:
            current_dialog = self.app.window(best_match=page[0])
            self.dialogs = [current_dialog]
            return self.dialog(page[1:])


def menu_select(dialog, step):
    element = step['element']
    try:
        dialog.menu_select(element)
    except:
        for el in element.split('->'):
            dialog.child_window(best_match=el).select()


def select(dialog, step):
    element = step['element']
    if dialog.backend.name == 'win32':
        dialog.window(best_match=element).select()
    elif dialog.backend.name == 'uia':
        dialog.child_window(best_match=element).select()


def click(dialog, step):
    element = step['element']
    if dialog.backend.name == 'win32':
        dialog.window(best_match=element).click_input()
    elif dialog.backend.name == 'uia':
        dialog.child_window(best_match=element).click_input()


def check_off(dialog, step):
    element = step['element']
    if dialog.backend.name == 'win32':
        dialog.window(best_match=element).check()
    elif dialog.backend.name == 'uia':
        dialog.child_window(best_match=element).check()


def double_click(dialog,step):
    element = step['element']
    if dialog.backend.name == 'win32':
        dialog.window(best_match=element).double_click_input()
    elif dialog.backend.name == 'uia':
        dialog.child_window(best_match=element).double_click_input()    


def input(dialog, step):
    element = step['element']
    vaule = step['data']['text']
    if dialog.backend.name == 'win32':
        dialog.window(best_match=element).type_keys(vaule, with_spaces=True, with_newlines='\r\n')
    elif dialog.backend.name == 'uia':
        dialog.child_window(best_match=element).type_keys(vaule, with_spaces=True, with_newlines='\r\n')


def set_text(dialog, step):
    element = step['element']
    vaule = step['data']['text']
    if dialog.backend.name == 'win32':
        dialog.window(best_match=element).set_edit_text(vaule)
    elif dialog.backend.name == 'uia':
        dialog.child_window(best_match=element).set_edit_text(vaule)


def send_keys(dialog, step):
    element = step['element']
    vaule = step['data'].get('text')
    dialog.set_focus()
    if element:
        if dialog.backend.name == 'win32':
            dialog.window(best_match=element).set_focus()
        elif dialog.backend.name == 'uia':
            dialog.child_window(best_match=element).set_focus()
        sendkeys(vaule)
    else:
        sendkeys(vaule)


def check(dialog, step):
    element = step['element']
    data = step['data']
    if not data:
        data = step['expected']
    output = step['output']
    for key in data:
        # 预期结果
        expected = data[key]
        # 切片操作处理
        s = re.findall(r'\[.*?\]', key)
        if s:
            s = s[0]
            key = key.replace(s, '')

        if key == 'text':
            if dialog.backend.name == 'win32':
                real = dialog.window(best_match=element).texts()[0].replace('\r\n', '\n')
            elif dialog.backend.name == 'uia':
                real = dialog.child_window(best_match=element).texts()[0].replace('\r\n', '\n')
        elif key == 'value':
            if dialog.backend.name == 'win32':
                real = dialog.window(best_match=element).text_block().replace('\r\n', '\n')
            elif dialog.backend.name == 'uia':
                real = dialog.child_window(best_match=element).get_value().replace('\r\n', '\n')                                         
        if s:
            real = eval('real' + s)

        if key == 'selected':
            if dialog.backend.name == 'win32':
                real = dialog.window(best_match=element).is_selected()
            elif dialog.backend.name == 'uia':
                real = dialog.child_window(best_match=element).is_selected()          
        elif key == 'checked':
            if dialog.backend.name == 'win32':
                real = dialog.window(best_match=element).is_checked()
            elif dialog.backend.name == 'uia':
                real = dialog.child_window(best_match=element).is_checked()
        elif key == 'enabled':
            if dialog.backend.name == 'win32':
                real = dialog.window(best_match=element).is_enabled()
            elif dialog.backend.name == 'uia':
                real = dialog.child_window(best_match=element).is_enabled()
        elif key == 'visible':
            if dialog.backend.name == 'win32':
                real = dialog.window(best_match=element).is_visible()
            elif dialog.backend.name == 'uia':
                real = dialog.child_window(best_match=element).is_visible()                
        elif key == 'focused':
            if dialog.backend.name == 'win32':
                real = dialog.window(best_match=element).is_focused()
            elif dialog.backend.name == 'uia':
                real = dialog.child_window(best_match=element).is_focused() 

        logger.info('DATA:%s' % repr(expected))
        logger.info('REAL:%s' % repr(real))
        compare(expected, real)

    # 获取元素其他属性
    for key in output:
        k = output[key]
        if dialog.window(best_match=element).class_name() == 'Edit' and k == 'text':
            k = 'value'
                        
        if k == 'text':
            if dialog.backend.name == 'win32':
                g.var[key] = dialog.window(best_match=element).texts()[0].replace('\r\n', '\n') 
            elif dialog.backend.name == 'uia':
                g.var[key] = dialog.child_window(best_match=element).texts()[0].replace('\r\n', '\n')

        if k == 'value':
            if dialog.backend.name == 'win32':
                g.var[key] = dialog.window(best_match=element).text_block().replace('\r\n', '\n') 
            elif dialog.backend.name == 'uia':
                g.var[key] = dialog.child_window(best_match=element).get_value().replace('\r\n', '\n')   


def window(dialog, step):
    element = step['element']
    if element.lower() in ('最小化', 'minimize'):
        dialog.minimize()
    elif element.lower() in ('最大化', 'maximize'):
        dialog.maximize()
    elif element.lower() in ('恢复', 'restore'):
        dialog.restore()          
    elif element.lower() in ('关闭', 'close'):
        dialog.close()
    elif element.lower() in ('前台', 'set_focus'):
        dialog.set_focus()
    elif element.lower() in ('重置', 'reset'):
        for i in range(10):
            if not dialog.has_focus():
                try:
                    # 如果弹出保存窗口
                    save = dialog.get_active()
                    element = '(N)'
                    if dialog.backend.name == 'win32':
                        save.window(best_match=element).click_input()
                    elif dialog.backend.name == 'uia':
                        save.child_window(best_match=element).click_input()                
                except:
                    pass
                #按 Alt+F4 关闭窗口
                sendkeys('%{F4}')
            else:
                break
        else:
            raise Exception(f'Reset the Windows is failure: try Alt+F4 for {i+1} times')


# coding:utf-8

# @Time: 2020/7/8 11:05
# @Auther: liyubin


import os
import json
import time
import datetime
import shutil
from injson import check
from super_sweetest.globals import g
from super_sweetest.elements import e
from super_sweetest.log import logger
from super_sweetest.utility import json2dict
from super_sweetest.servers.common import read_data, write_data


"""
mqtt keyword
"""


def login_mqtt(step):
    """登录mqtt"""
    request('login_mqtt', step)


def send(step):
    """发送mqtt消息"""
    request('mqtt', step)


def send_mqtt(theme_dict, data):
    """发送mqtt主方法"""

    # 主题信息 topic/qos
    qos = int(theme_dict.get('QOS', 1))
    # 订阅主题最后 # 会被替换
    SUBSCRIBE = theme_dict.get('SUBSCRIBE', '') + '#'
    PUBLISH = theme_dict.get('PUBLISH', '')


    if data != {} and SUBSCRIBE and PUBLISH:

        # 自定义header，方便测试服务器捕获访问记录
        data.update({'user-agent': 'AutoTest-MQTT'})
        # 测试数据 msg
        msg = json.dumps(data, ensure_ascii=False)

        logger.info('Send Mqtt Msg: %s' % msg)
        # 订阅
        g.mqtt_client_.on_subscribe(topic=SUBSCRIBE, qos=qos)
        # 发布消息
        g.mqtt_client_.on_publish(topic=PUBLISH, msg=msg, qos=qos)
        # 消息回调 / on_message_come 内部添加 返回值处理 的函数
        g.mqtt_client_.on_message()
        logger.info(' --- 等待消息回调 --- ')
    else:
        data_msg = '当前主题：%s \n请求体: %s' % (str(theme_dict), str(data))
        theme_msg = '当前主题：%s 参数有误\n正确格式：{"SUBSCRIBE": "订阅的主题", "PUBLISH":"发布的主题", "QOS": 数值}' % str(theme_dict)
        raise Exception(theme_msg if not SUBSCRIBE and PUBLISH else data_msg)


def request(kw, step):
    """
    登录/发送mqtt请求/数据处理
    :param kw:
    :param step:
    :return:
    """
    # 全局文件路径
    global g_file
    g_file = os.path.join('log', microsecond() + '.json')

    element = step['element']
    # 订阅/发布的主题
    theme = e.get(element)[1]
    theme_dict = json2dict(theme.replace('，', ',').replace('“', '"')) if '{' in theme else ''

    data = step['data']
    # 测试数据解析时，会默认添加一个 text 键，需要删除
    if 'text' in data and not data['text']:
        data.pop('text')

    _data = {}

    if kw in ('mqtt', 'login_mqtt'):
        _data['json'] = json2dict(data.pop('json', '{}'))

    for k in data:
        for s in ('{', '[', 'False', 'True'):
            if s in data[k]:
                try:
                    data[k] = eval(data[k])
                except:
                    logger.warning('Try eval data failure: %s' % data[k])
                break

    expected_ = step['expected']
    expected_['status_code'] = expected_.get('status_code', None)
    expected_['json'] = json2dict(expected_.get('json', '{}'))
    timeout = float(expected_.get('timeout', 8))
    expected_['time'] = float(expected_.get('time', 0))

    global filter_str
    filter_str = expected_.get('filter', ':') # 过滤mqtt回调消息中包含的

    # 登录/初始化mqtt客户端
    if kw == 'login_mqtt' and g.mqtt_client_ == '':
        # 登录时再导入,避免mqtt_client中导入write_mqtt_msg引起的循环导入错误
        from super_sweetest.servers.mqtt_client import Mqtt_Init
        Mqtt_Init(_data['json']).setup_mqttclient()

    # 发送mqtt消息
    elif kw == 'mqtt':
        send_mqtt(theme_dict, _data['json'])

        # 运行结果预处理
        get_response_expect_var(expected_, step, timeout)


def shutil_file():
    """
    删除微秒级文件
    """
    if os.path.exists(g_file):
        try:
            shutil.rmtree(g_file)
        except:
            try:
                os.remove(g_file)
            except:
                logger.info('remove file: %s error' % g_file)


def microsecond():
    """微秒级时间戳"""
    return str(round(time.time() * 1000000))


def write_mqtt_msg(message):
    """
    消息回调调用，写入消息回调
    :param message:
    :return:
    """
    if filter_str in message:
        write_data(g_file, message, '', 'w+') if message else None
        logger.info('filter_str:  ' + filter_str)


def read_mqtt_msg(timeout=8):
    """
    获取mqtt消息回调
    :param timeout: 超时
    :return:
    """
    # 隐式等待/提高消息回调效率
    now_time_flag = datetime.datetime.now()
    while True:
        time.sleep(0.01)
        if (datetime.datetime.now() - now_time_flag).seconds > timeout:
            break
        if os.path.exists(g_file):
            data = read_data(g_file, '', 'r+')
            if data: # 读取内容存在
                return data
    return '{"msg": "MQTT未收到消息回调"}'


def get_response_expect_var(expected_, step_, timeout):
    """
    获取消息回调中写入的返回值
    预期值处理
    变量处理
    :param response:
    :return:
    """
    # 读取返回信息
    response_ = read_mqtt_msg(timeout)
    logger.info("mqtt response : %s" %response_)

    # 清理生成的文件
    shutil_file()

    # 消息回调转dict
    response = json2dict(response_)
    var = {}  # 存储所有输出变量

    if expected_['json']:
        result = check(expected_['json'], response)
        logger.info('json check result: %s' % result)
        if result['code'] != 0:
            raise Exception(f'json | EXPECTED:{repr(expected_["json"])}\nREAL:{repr(response)}\nRESULT: {result}')
        elif result['var']:
            var = dict(var, **result['var'])
            g.var = dict(g.var, **result['var'])
            logger.info('json var: %s' % (repr(result['var'])))

    output = step_['output']
    if output:
        logger.info('output: %s' % repr(output))

    for k, v in output.items():
        if k == 'json':
            sub = json2dict(output.get('json', '{}'))
            result = check(sub, response)
            # logger.info('Compare json result: %s' % result)
            var = dict(var, **result['var'])
            g.var = dict(g.var, **result['var'])
            logger.info('json var: %s' % (repr(result['var'])))

    if var:
        step_['_output'] += '\n||output=' + str(var)



# coding:utf-8

# @Time: 2020/10/28 11:44
# @Auther: liyubin

import os
import json
import requests
from super_sweetest.config import URL_LDS_UI, URL_BAIDU_AIP

"""
人工智能 AI识别 UI/UI文本
UI     基于百度AI平台，训练素材
UI文本 基于百度AIP识别高精度接口
"""


def recognition_ui(input_file):
    """
    UI识别正确和异常
    :param input_file:
    :return:
    """
    file = {'file': open(input_file, 'rb')}
    data = {'code': 117}
    res = requests.post(URL_LDS_UI, data=data, files=file).json()
    return res


def get_ui_results(response):
    """
    获取UI识别中识别结果
    :param response:
    :return:
    """
    return response.get('msg', '')


def recognition_text(input_file, language):
    """
    UI文本识别
    :param input_file:
    :param language: app_config.json 设定的语言,增加识别文本正确性
    :return:
    """
    # 默认用户从 py 导入， 当请求次数上限写入新的用户名并完成重试一次
    config_name_file = 'config_name.py'
    if not os.path.exists(config_name_file):
        with open(config_name_file, 'w+')as fp:
            fp.write('CONFIG_NAME = ""')

    CONFIG_NAME = get_config_name()
    file = {'file': open(input_file, 'rb')}  # 发送后接口中通过file.read()获取值
    OPTIONS = {}
    OPTIONS["language_type"] = 'auto_detect' if language is 'CHN_ENG' else language  # 当设置中英文时，用auto_detect，识别效果好
    OPTIONS["detect_direction"] = "true"
    OPTIONS["detect_language"] = "true"
    OPTIONS["probability"] = "true"
    data = {
            'code': 117,
            'config_name': CONFIG_NAME,
            'options': json.dumps(OPTIONS)
            }
    res = requests.post(url=URL_BAIDU_AIP, data=data, files=file).json()

    # 超过每日限量, 通过 config_name 换百度账号
    error_msg = res.get('error_msg', '')
    if 'limit reached' in error_msg:
        write_config_name = 'yang' if CONFIG_NAME == '' else ''  # 每次请求不同时可以写入不同的 config_name
        with open(config_name_file, 'w+')as fp:
            fp.write('CONFIG_NAME = "{}"'.format(write_config_name))  # 写入切换后的
        # 尝试切换一个账户
        data['config_name'] = write_config_name
        res = requests.post(url=URL_BAIDU_AIP, data=data, files={'file': open(input_file, 'rb')}).json()
        return res
    return res


def get_config_name():
    """
    导入config_name
    避免文件不存在导入报错
    :return:
    """
    from config_name import CONFIG_NAME
    return CONFIG_NAME


def get_words(response):
    """
    获取aip接口返回中的words
    :param response:
    :return:
    """
    if 'words_result' in response.keys():
        new_worlds = []
        for words in response.get('words_result', []):
            new_worlds.append(words.get('words'))
        return new_worlds
    # 超过每日限量
    error_msg = response.get('error_msg')
    assert 'limit reached' not in error_msg, '⚠ ⚠ ⚠ ⚠ 文本识别调用次数超过当日限量...'


def get_screen_text(file_path, language):
    """
    获取截图中文本信息
    :return:
    """
    from super_sweetest.snapshot import get_air_screenshot # 防止初始化时 循环导入报错，导入写在这里
    assert str(file_path).endswith('.png'), '截图路径必须以.png结尾'
    get_air_screenshot(file_path=file_path)
    response = recognition_text(file_path, language)
    return get_words(response)



"""
通过unicode编码范围判断字符
"""


def is_in_copywriting(words, copywriting_data):
    """
    检查单词词语是否在文案中
    直接检查识别后数据，不对数据替换标点符号
    :param words:
    :param copywriting_data:
    :return:
    """
    if words in copywriting_data:
        return True
    return False


def replace_punctuate(words, uchar=False, baidu_aip=False):
    """
    str中标点符号替换为'' 适合unicode编码范围判断单个字符
    :param words:
    :param uchar: 忽略所有符号
    :param baidu_aip: 忽略百度识别后异常的符号
    :return:
    """
    punctuate_list_uchar = [' ', '`', '~', '@', '!', '#', '$', '%', '^', '&', '*', '(', ')', '_', '-', '=', '+', '/',
                            '|', '[',
                            ']', '{', '}', '.', ',', '，', '。', '?', '、', '<', '>', '《', '》', '！', '·', '（',
                            '）', '【', '】', ':', ';', '；', '"', '’', '”']
    punctuate_list_officers = ['<', '>', '>>>>']  # 忽略baidu_aip识别后有误的标点
    if uchar:
        punctuate_list = punctuate_list_uchar
    elif baidu_aip:
        punctuate_list = punctuate_list_officers
    else:
        punctuate_list = ''
    new_words = ''
    new_words_list = []
    for word in words:
        if word not in punctuate_list:
            new_words = new_words + word if uchar else None
            new_words_list.append(word)
    return new_words if uchar else new_words_list


def is_chinese(uchar):
    """
    是否为中文
    """
    if u'\u4e00' <= uchar <= u'\u9fa5':
        return True
    return False


def is_number(uchar):
    """
    是否为数字
    """
    if u'\u0030' <= uchar <= u'\u0039':
        return True
    return False


def is_english(uchar):
    """
    是否为英文字母
    """
    if u'\u0041' <= uchar <= u'\u005a' or u'\u0061' <= uchar <= u'\u007a':
        return True
    return False


def is_other(uchar):
    """
    判断是否非 汉字/数字/英文 字符
    """
    if not (is_chinese(uchar) or is_number(uchar) or is_english(uchar)):
        return True
    return False


def is_japan(uchar):
    """
    是否为日文
    """
    if u'\u0800' <= uchar <= u'\u4e00 ':
        return True
    return False


def is_korean(uchar):
    """
    是否为韩文
    """
    if u'\uAC00' <= uchar <= u'\uD7A3':
        return True
    return False


def is_France_and_german(uchar):
    """
    德文或法文
    :param uchar:
    :return:
    """
    if u'\u00C0' <= uchar <= u'\u00FF':
        return True
    return False


def is_russian(uchar):
    """
    是否为俄语
    """
    if u'\u0400' <= uchar <= u'\u052f':
        return True
    return False




def is_language(word_list):
    """
    通过文本判断和输出是那种语言
    :return:
    """
    language = []
    for i in range(len(word_list)):
        uchar_ = word_list[i].replace(' ', '')
        if is_english(uchar_):
            language.append('en')
        elif is_chinese(uchar_):
            language.append('zh')
        elif is_russian(uchar_):
            language.append('ru')
        elif is_korean(uchar_):
            language.append('kr')
        elif is_japan(uchar_):
            language.append('ja')
        else:
            language.append(uchar_)
    return max(language, key=language.count)  # 获取list出现次数最多的元素





# coding:utf-8

import os
import sys
import time
import shutil
import json
import configparser
import requests
import redis
from git import Repo
import platform
import datetime
from super_sweetest.servers.common import write_data, read_data


class MqttRun:
    """
    mqtt自动化测试执行端
    """

    def __init__(self, run_data):
        # 工作空间名
        self.NAME = run_data['name']
        # 平台URL "".join(map(chr, run_data['platform_url']))
        self.PLATFORM_URL = "".join(map(chr, json.loads(run_data['platform_url'])))
        # 队列IP、队列KEY、队列端口号
        self.REDIS_IP = "".join(map(chr, json.loads(run_data['redis_ip'])))
        self.REDIS_KEY = int("".join(map(chr, json.loads(run_data['redis_key']))))
        self.REDIS_PORT = int("".join(map(chr, json.loads(run_data['redis_port']))))
        # GIT仓库URL
        self.GIT_URL = "".join(map(chr, json.loads(run_data['git_url'])))
        if self.GIT_URL.count('@') - 1 != 0:
            self.GIT_URL = self.GIT_URL.replace('@', '%40', self.GIT_URL.count('@') - 1)
        # 执行端注册ID
        self.EXECUTION_ID = int(run_data['execution_id'])
        # 通过队列IP、队列KEY、队列端口号参数连接Redis数据库
        self.POOL = redis.ConnectionPool(host=self.REDIS_IP, port=self.REDIS_PORT, db=self.REDIS_KEY,
                                         decode_responses=True)
        self.REDIS_POOL = redis.Redis(connection_pool=self.POOL)
        # 初始日志文件的起始行号
        self.initial_log_line = 0
        # 初始日志文件的日期前缀
        self.initial_log_time = ''

    @staticmethod
    def mkdir_and_del(src_path):
        """
        删除旧文件夹再创建新文件夹
        """
        if os.path.isdir(src_path):
            shutil.rmtree(src_path)
        os.mkdir(src_path)

    @staticmethod
    def reset_file_tree(file_name, project_name, test_environment):
        """
        将Git用例仓库里的project_name项目目录中的file_name文件复制到执行目录中
        """
        case_path = os.path.join(os.getcwd(), "git_case")
        lib_dst_path = os.path.join(
            case_path, project_name + "-Project-mqtt", test_environment, file_name)
        # 确定Git用例存储库中是否有一个lib_dst_path目录
        if os.path.isdir(lib_dst_path):
            lib_src_path = os.path.join(os.getcwd(), file_name)
            # 确定执行目录下是否有一个lib_src_path目录
            if os.path.isdir(lib_src_path):
                shutil.rmtree(lib_src_path)
            shutil.copytree(lib_dst_path, lib_src_path)

    def pull_case(self):
        """
        clone（拉取）和pull（更新）测试用例仓库的内容
        """
        try:
            git_case_path = os.path.join(os.getcwd(), "git_case")
            if os.path.isdir(git_case_path):
                print("[mqtt Server Run Info]> 发现 git_case 目录, 执行更新最新测试用例流程")
                repo = Repo.init(path=git_case_path)
                remote = repo.remote()
                remote.pull()
                print("[mqtt Server Run Info]> 已更新 GitLab 仓库中 git_case 项目的测试用例")
            else:
                print("[mqtt Server Run Info]> 找不到 git_case 目录, 执行下载新测试用例流程")
                Repo.clone_from(url=self.GIT_URL, to_path=git_case_path)
                print("[mqtt Server Run Info]> 已下载 GitLab 仓库中 git_case 项目的测试用例")
        except:
            pass

    def all_mkdir_copy(self, test_project, test_environment):
        """
        复制指定测试项目、测试类型、测试环境中的相关目录到自动化运行目录下
        """
        self.reset_file_tree(file_name="lib", project_name=test_project, test_environment=test_environment)
        self.reset_file_tree(file_name="data", project_name=test_project, test_environment=test_environment)
        self.reset_file_tree(file_name="element", project_name=test_project, test_environment=test_environment)
        self.reset_file_tree(file_name="testcase", project_name=test_project, test_environment=test_environment)
        self.reset_file_tree(file_name="files", project_name=test_project, test_environment=test_environment)
        MqttRun.mkdir_and_del(src_path=os.path.join(os.getcwd(), "JUnit"))
        MqttRun.mkdir_and_del(src_path=os.path.join(os.getcwd(), "report"))
        MqttRun.mkdir_and_del(src_path=os.path.join(os.getcwd(), "details"))


    def write_testdata_and_run(self, test_name):
        """
         写入测试所需数据
        :param test_name:
        """
        if test_name:
            sys = platform.system()  # 操作系统
            # MQTT
            test_case_data = {'test_name': test_name, 'status': False}
            write_data('./run_case_data.json', test_case_data, 'json', 'w+')
            if sys.lower() in ('windows'):
                # windows 启动test_case模块
                os.popen('start python ./test_case.py')
            else:
                # 其他系统启动
                os.system('nohup python3 ./test_case.py &')

        # 如果运行结束状态status：True，继续下一步
        # 隐式等待/避免执行端假死
        now_time_flag = datetime.datetime.now()
        while True:
            time.sleep(5)
            if (datetime.datetime.now() - now_time_flag).seconds > 3600:
                break
            else:
                status = read_data('./run_case_data.json', 'json', 'r')
                if status['status']:
                    return status.get('msg', '')


    def read_log_file(self):
        """
        从以日期为维度统计的日志中提取出本次的日志文件，保存至log/new.log
        """
        # 如果日志文件的日期前缀不是当天的日期
        if self.initial_log_time != time.strftime('%Y%m%d'):
            # 重置日志文件的前缀、起始行数
            self.initial_log_time = time.strftime('%Y%m%d')
            self.initial_log_line = 0
        # 读取日志文件的全部字符串
        with open(os.path.join(os.getcwd(), 'log', time.strftime('%Y%m%d.log')), "r", encoding='utf-8') as f:
            # 将日志字符串按换行符拆成列表
            log_file_data = f.read().split('\n')
        # 如果起始日志行反而比最大行数都大的话
        if self.initial_log_line > len(log_file_data):
            # 重置日志文件的起始行数
            self.initial_log_line = 0
        # 按行号增量取出本次测试的日志
        log_file_list = log_file_data[self.initial_log_line:]
        # 更新日志文件的起始行号
        self.initial_log_line = len(log_file_data)
        # 存储日志信息的字典
        log_file_dict = {}
        # 用这个变量控制每一行都是不重复的
        previous_row = ''
        # 整理每一行日志
        for i in range(len(log_file_list)):
            # 不读取空行
            if log_file_list[i] == '':
                continue
            # 不读取和上一行重复的数据
            if log_file_list[i] == previous_row:
                continue
            # 判断当前行是否包含特殊分割符
            if ']: #  ' in log_file_list[i]:
                # 将日志信息按用途分类
                log_file_dict[i] = {
                    'time': log_file_list[i][:23],
                    'grade': log_file_list[i][25:log_file_list[i].find(']')],
                    'message': log_file_list[i][log_file_list[i].find(']')+6:],
                }
            else:
                # 如果是错误信息，上传特殊格式
                log_file_dict[i] = {
                    'time': '',
                    'grade': '',
                    'message': log_file_list[i].replace(' ', '&nbsp;'),
                }
            # 更新上一行的数据
            previous_row = log_file_list[i]
        # 在log目录下写入新的日志文件
        with open(os.path.join('log', 'new.log'), "w", encoding='utf-8') as file:
            file.write(json.dumps(log_file_dict))

    def post_heartbeat(self, status, localtime=None, task_dict=None, remark=None):
        """
        # 描述:
            发送心跳请求到ATP服务器（失败重试，直至恢复连接）
        # 用法0（第一次）:
            self.post_heartbeat(status=0)
        # 用法1（正常）:
            self.post_heartbeat(status=1, localtime=[调用时间])
        # 用法2（任务无法完成/放弃任务，需要在备注中提交错误信息）:
            self.post_heartbeat(status=2, localtime=[调用时间], task_dict=[任务字典], remark=[备注])
        # 用法3（开始执行任务）:
            self.post_heartbeat(status=3, localtime=[调用时间], task_dict=[任务字典])
        # 用法4（任务执行完成，上传测试报告）:
            self.post_heartbeat(status=4, localtime=[调用时间], task_dict=[任务字典])
        # 用法5（执行端异常，需要在备注中提交异常信息）:
            self.post_heartbeat(status=5, remark=[备注])
        # 用法6（执行端离线）:
            self.post_heartbeat(status=6)
        # 用法7（执行端变更信息，需要在备注中提交变更信息字典字符串）:
            self.post_heartbeat(status=7, remark=[备注])
        """
        if not localtime:
            localtime = time.asctime(time.localtime(time.time()))
        # ATP平台端上执行端心跳的mqtt接口
        heartbeat_mqtt_url = self.PLATFORM_URL + 'software/execution/heartbeat/'
        # 心跳请求连接次数
        connect = 0
        # 开始发送心跳请求（不成功便成仁，真男人从不回头看爆炸）
        while True:
            # 捕获异常
            try:
                # 执行端心跳参数
                up_data = {'id': self.EXECUTION_ID, 'status': status, 'task_dict': json.dumps(task_dict),
                           'remark': remark}
                # 状态为4时，组织待上传的次数报告文件
                if status == 4:
                    # 拔高变量的层级
                    files_report = None
                    files_junit = None
                    # 取出XXX-Report@XXXXXX_XXX.xlsx的测试报告文件名
                    for _root, _dirs, files in os.walk(os.path.join('report', task_dict['test_name'])):
                        files_report = files[0]
                    # 取出XXX-Report@XXXXXX_XXX.xml的测试汇总文件名
                    for _root, _dirs, files in os.walk('JUnit'):
                        files_junit = files[0]
                    # 读取测试报告文件
                    with open(os.path.join('report', task_dict['test_name'], files_report), 'rb') as fp1:
                        files_report_obj = fp1.read()
                    # 读取测试汇总文件
                    with open(os.path.join('JUnit', files_junit), 'rb') as fp2:
                        files_junit_obj = fp2.read()
                    # 读取测试详情文件
                    with open(os.path.join('details', 'details.txt'), 'rb') as fp3:
                        files_details_obj = fp3.read()
                    # 读取测试日志文件
                    self.read_log_file()
                    with open(os.path.join('log', 'new.log'), 'rb') as fp4:
                        files_log_obj = fp4.read()
                    # 汇总XXX-Report@XXXXXX_XXX.xlsx、XXX-Report@XXXXXX_XXX.xml、details.txt、new.log文件对象
                    files_obj = {
                        'file_obj': (str(task_dict['queue_id']) + '.xlsx', files_report_obj),
                        'file_obj_junit': (str(task_dict['queue_id']) + '.xml', files_junit_obj),
                        'file_obj_details': (str(task_dict['queue_id']) + '.txt', files_details_obj),
                        'file_obj_log': (str(task_dict['queue_id']) + '.log', files_log_obj)
                    }
                else:
                    files_obj = {}
                # 向ATP平台端发送执行端心跳请求
                requests_data = requests.post(heartbeat_mqtt_url, data=up_data, files=files_obj)
                requests_json = requests_data.json()
                # 判断平台端心跳状态处理结果
                if requests_json['code'] == 200:
                    # 心跳请求响应成功，结束死循环
                    print('[Heartbeat][' + localtime + ']> ' + str(requests_json['data']))
                    break
                # 服务器内部错误错误
                else:
                    print('[Heartbeat Error][' + localtime + ']> > ' + str(requests_json))
            # 网络请求错误
            except Exception as exc:
                print('[Heartbeat Error][' + localtime + ']> ' + str(exc))
            # 30秒后重新发送心跳请求
            time.sleep(30)
            connect += 1
            print('[Heartbeat Error][' + localtime + ']> 进行第 ' + str(connect) + ' 次重新连接')
            # 防止 异常后文件找不到 进入死循环 join argument must be str or bytes, not 'NoneType'
            if connect > 2:
                break


    def run(self):
        self.mkdir_and_del(src_path=os.path.join(os.getcwd(), "log"))
        self.mkdir_and_del(src_path=os.path.join(os.getcwd(), "snapshot"))
        self.pull_case()
        print('[mqtt Server Run Info]> 检测是否有历史任务未完成')
        # 上报第一次/刚启动执行端的心跳
        self.post_heartbeat(status=0)
        while True:
            time.sleep(10)
            try:
                # 获取Redis任务队列的信息长度
                length = self.REDIS_POOL.llen(self.EXECUTION_ID)
                # 生成当前时间字符串，用于打印输出
                localtime = time.asctime(time.localtime(time.time()))
                # 如果Redis任务队列的信息长度小于0，那就说明没有任务
                if length > 0:
                    print('[mqtt Server Run Info][' + localtime + ']> ' + '发现新的任务信息')
                    self.pull_case()
                    value_dict = json.loads(self.REDIS_POOL.lpop(self.EXECUTION_ID))
                    # 上报开始执行任务的心跳
                    self.post_heartbeat(status=3, localtime=localtime, task_dict=value_dict)
                    try:
                        print('[Queue Task Dict]> ' + str(value_dict))
                        print('[mqtt Server Run Info]> 重置 X-Sweetest 测试运行环境')
                        self.all_mkdir_copy(test_project=value_dict['test_project'],
                                            test_environment=value_dict['test_environment'])
                        print('[mqtt Server Run Info]> X-Sweetest 测试运行环境就绪')
                        self.write_testdata_and_run(test_name=value_dict['test_name'])
                        print('[mqtt Server Run Info]> X-Sweetest 测试完成并上传测试报告')
                        # 上报任务执行完成的心跳
                        self.post_heartbeat(status=4, localtime=localtime, task_dict=value_dict)
                        print('[mqtt Server Run Info]> 测试报告上传完毕')
                    except Exception as exc:
                        # 上报任务无法完成的心跳
                        self.post_heartbeat(status=2, localtime=localtime, task_dict=value_dict, remark=str(exc))
                        print('[mqtt Server Run Error]> ' + str(exc))
                else:
                    # 上报正常的心跳
                    self.post_heartbeat(status=1, localtime=localtime)
                # 调用Flush，从内存中刷入日志文件
                sys.stdout.flush()
            except Exception as exc:
                # 上报执行端异常的心跳
                self.post_heartbeat(status=5, remark=str(exc))
                print('[mqtt Server Run Error]> ' + str(exc))


def main_():
    """
    主函数
    :return:
    """
    # 读取工作空间目录下的config配置文件
    config_file = 'config.ini'
    config = configparser.RawConfigParser()
    config.read(config_file, encoding='utf-8')
    # 读取配置中的ATP-Server内容
    data = dict(config.items('ATP-Server'))
    # 创建mqtt自动化测试执行端对象
    r = MqttRun(run_data=data)
    # 启动mqtt自动化测试执行端
    r.run()



# coding=utf-8

# @Time: 2020/3/5 10:57
# @Auther: liyubin

import os
import time
import re

"""
appium server
get android mobile desc
"""


def localtime():
    """获取当前时间"""
    localtime_ = time.asctime(time.localtime(time.time()))
    return localtime_


def get_devices():
    """
    获取已连接的设备名称
    :return: list
    """
    device_list = []

    adb_devices = 'adb devices'
    with os.popen(adb_devices, 'r')as fp:
        devices_texts = fp.readlines()

    for device in devices_texts:
        device_ = device.strip('\n').split(' ')[0].replace('device', '').replace('\t', '')
        if len(device_) > 0 and device_ != 'List' and 'offline' not in device_:
            device_list.append(device_)
    print(localtime() + ' --- device list online: {} --- '.format(device_list))
    return device_list


def get_resolution(devices_name):
    """
    获取单个手机分辨率
    return: tuple
    """
    adb_resolution = 'adb -s {} shell wm size'.format(devices_name)
    with os.popen(adb_resolution, 'r')as fp:
        resolution_texts = fp.readlines()[0]

    resol = resolution_texts.replace('Physical size: ', '').split('x')

    resolution = tuple((int(resol[0]), int(resol[1])))
    # print(localtime() + '--- resolution is : {} ---'.format(resolution))
    return resolution


def get_mobile_name(device_name):
    """
    获取单个手机名称
    :param device_name:
    :return: 名称
    """
    adb_mobile = 'adb -s {} shell getprop ro.product.model'.format(device_name)
    with os.popen(adb_mobile, 'r')as fp:
        mobile_texts = fp.readlines()[0]
    mobile = mobile_texts.strip('\n')
    # print(localtime() + '--- mobile is : {} ---'.format(mobile))
    return mobile


def get_physical_density(device_name):
    """
    获取屏幕光学密度
    :param device_name:
    :return: 分辨率值
    """
    adb_physical_density = 'adb -s {} shell wm density'.format(device_name)
    with os.popen(adb_physical_density, 'r')as fp:
        physical_density_texts = fp.readlines()[0]
    physical_density = physical_density_texts.replace('Physical density: ', '').strip('\n')
    # print(localtime() + '--- physical_density is : {} ---'.format(physical_density))
    return physical_density


def get_platform_version(device_name):
    """
    获取手机系统版本
    :param device_name:
    :return: 版本号
    """
    adb_platform_version = 'adb -s {} shell getprop ro.build.version.release'.format(device_name)
    with os.popen(adb_platform_version, 'r')as fp:
        platform_version_texts = fp.readlines()[0]
    platform_version = platform_version_texts.strip('\n')
    # print(localtime() + '--- mobile is : {} ---'.format(mobile))
    return platform_version


# 获取手机信息主方法
def get_mobile_desc(devices_list):
    """
    获取手机的详细信息
    :param devices_list:
    :return: 手机详情[{k:v},{k:v}]
    """
    mobile_desc_list = []
    for device_name in devices_list:

        # 如果未授权就直接反回device_name，不然获取不到设备信息
        if re.search('unauthorized', device_name):
            mobile_desc_list.append({'device_name': device_name, 'mobile_name': device_name})
        else:
            # device_name # 设备名称
            mobile_name = get_mobile_name(device_name)  # 手机名称
            resolution = get_resolution(device_name)  # 分辨率
            physical_density = get_physical_density(device_name)  # 屏幕密度
            platform_version = get_platform_version(device_name)  # 系统版本

            mobile_desc_list.append({'mobile_name': mobile_name, 'device_name': device_name, 'resolution': resolution,
                                     'physical_density': physical_density, 'platform_version': platform_version})

    # print(localtime() + ' --- device list online: {} --- '.format(mobile_desc_list))
    return mobile_desc_list


# appium相关
################################
def find_port_kill_pid(port):
    """
    查找和杀端口的pid
    return: 返回pid循环判断kill
    """
    print(' --- find port %s  --- ' % port)
    find_port = 'netstat -aon | findstr %s' % port
    with os.popen(find_port, 'r')as fp:
        text = fp.read()
    pid = text[-6:-1]
    print(' --- kill port %s --- ' % port)
    kill_port = 'taskkill -f -pid %s' % pid
    cmd = os.popen(kill_port)
    cmd.close()
    return pid


def kill_all(port=None):
    """
    杀所有appium进程
    """
    if port:
        pid = find_port_kill_pid(port)
    else:
        pid = find_port_kill_pid(5037)  # 模拟器请注释
        # pid = ''
    print(' --- if pid is None kill success ：%s ? --- ' % pid)
    if not pid:
        return True
    for i in range(20):
        print(' --- kill: %s --- ' % pid)
        pid_ = find_port_kill_pid(port)
        if not pid_:
            break


def start_appium_server(port=None):
    """
    启动appium服务，在生成驱动前调用
    """
    if port:
        print(' --- Appium Server Start Port: %s --- ' % port)
        start_ = 'start appium -a 127.0.0.1 -p %s' % port
        cmd = os.popen(start_)
        cmd.close()
    else:
        start_4723 = 'start appium --no-reset --session-override --log ./appium_logs.log'
        print(' --- Appium Server Start Port: %s --- \n --- Create log --- ' % port)
        cmd1 = os.popen(start_4723)
        cmd1.close()
    time.sleep(7)


def stop_appium_server(port=None):
    """
    关闭appium服务，可在关闭驱动后调用
    """
    if port:
        print(' --- Stop Appium Server | Port: %s --- ' % port)
        kill_all(port)
    else:
        print(' --- Stop All Appium Server --- ')
        kill_node = 'taskkill /f /t /im node.exe'
        cmd = os.popen(kill_node)
        cmd.close()
    time.sleep(7)


def exist_port(port):
    """
    查询port 4723 是否被占用，占用就不重复启动appium
    return True 占用
    """
    find_port = 'netstat -aon | findstr %s' % port
    with os.popen(find_port)as fp:
        text = fp.read()
    if str(port) not in text:
        return False
    return True


def run_appium_main(port=4723):
    """
    主方法，占用不启动，未占用启动
    """
    status = exist_port(port)
    if not status:
        stop_appium_server(port)

        start_appium_server(port)
    else:
        print(' --- Appium Server Start Raining Now !--- ')



# 本文件为 Http 测试调用，请勿修改名称。

def before_send(method, data, kwargs):
    '''
    method: str, 请求类型 
    值可以为：post, get, put, patch, delete, options

    data: dict，格式如下：
        {'headers':{},
         'params': {},
         'data': {},
         'json': {},
         'files': 'file path'
        }

    keargs: dict，其他参数
    '''
    # handle the request here

    return data, kwargs

def after_receive(response):
    '''
    response: dict, 格式如下：
    {'status_code': 200,
     'headers': {},
     '_cookies': '',  # 原始内容
     'cookies': {},   # dict 格式
     'content': b'',
     'text': '',
     'json': {}
     }
    '''
    # handle the response here

    return response



from selenium.webdriver.common.action_chains import ActionChains
from selenium.common.exceptions import ElementClickInterceptedException
from time import sleep
import re
from super_sweetest.globals import g
from super_sweetest.elements import e
from super_sweetest.windows import w
from super_sweetest.locator import locating_element
from super_sweetest.log import logger
from super_sweetest.utility import compare, json2dict


class Common():
    @classmethod
    def title(cls, data, output):
        logger.info('DATA:%s' % repr(data['text']))
        logger.info('REAL:%s' % repr(g.driver.title))
        try:
            if data['text'].startswith('*'):
                assert data['text'][1:] in g.driver.title
            else:
                assert data['text'] == g.driver.title
        except:
            raise Exception(f'Check Failure, DATA:{data["text"]}, REAL:{g.driver.title}')
        # 只能获取到元素标题
        for key in output:
            g.var[key] = g.driver.title
        return g.driver.title


    @classmethod
    def current_url(cls, data, output):
        logger.info('DATA:%s' % repr(data['text']))
        logger.info('REAL:%s' % repr(g.driver.current_url))
        try:
            if data['text'].startswith('*'):
                assert data['text'][1:] in g.driver.current_url
            else:
                assert data['text'] == g.driver.current_url
        except:
            raise Exception(f'Check Failure, DATA:{data["text"]}, REAL:{g.driver.current_url}')            
        # 只能获取到元素 url
        for key in output:
            g.var[key] = g.driver.current_url
        return g.driver.current_url


def open(step):
    element = step['element']
    value = e.get(element)[1]
    if step['data'].get('清理缓存', '') or step['data'].get('clear', ''):
        g.driver.delete_all_cookies()
    if step['data'].get('打开方式', '') == '新标签页' or step['data'].get('mode', '').lower() == 'tab':
        js = "window.open('%s')" % value
        g.driver.execute_script(js)
        # 判断是否打开了新的窗口，并将新窗口添加到所有窗口列表里
        all_handles = g.driver.window_handles
        for handle in all_handles:
            if handle not in w.windows.values():
                w.register(step, handle)
    else:
        if step['data'].get('打开方式', '') == '新浏览器' or step['data'].get('mode', '').lower() == 'browser':
            w.close()
            g.set_driver()
            w.init()
        g.driver.get(value)
        w.open(step)
    cookie = step['data'].get('cookie', '')
    if cookie:
        g.driver.add_cookie(json2dict(cookie))
        co = g.driver.get_cookie(json2dict(cookie).get('name', ''))
        logger.info(f'cookie is add: {co}')
    sleep(0.5)


def check(step):
    data = step['data']
    if not data:
        data = step['expected']

    element = step['element']
    element_location = locating_element(element)
    if '#' in element:
        e_name = element.split('#')[0] + '#'
    else:
        e_name = element
    by = e.elements[e_name]['by']
    output = step['output']
    var = {}

    if by in ('title', 'current_url'):
        var[by] = getattr(Common, by)(data, output)

    else:
        for key in data:
            # 预期结果
            expected = data[key]
            # 切片操作处理
            s = re.findall(r'\[.*?\]', key)
            if s:
                s = s[0]
                key = key.replace(s, '')

            if key == 'text':
                real = element_location.text
            else:
                real = element_location.get_attribute(key)
            if s:
                real = eval('real' + s)

            logger.info('DATA:%s' % repr(expected))
            logger.info('REAL:%s' % repr(real))
            try:
                compare(expected, real)
            except:
                raise Exception(f'Check Failure, DATA:{repr(expected)}, REAL:{repr(real)}')

        # 获取元素其他属性
        for key in output:
            if output[key] == 'text':
                var[key] = g.var[key] = element_location.text
            elif output[key] in ('text…', 'text...'):
                if element_location.text.endswith('...'):
                    var[key] = g.var[key] = element_location.text[:-3]
                else:
                    var[key] = g.var[key] = element_location.text
            else:
                var[key] = g.var[key] = element_location.get_attribute(output[key])
    if var:
        step['_output'] += '\n||output=' + str(var)
    return element_location


def notcheck(step):
    data = step['data']
    if not data:
        data = step['expected']

    element = step['element']
    # element_location = locating_element(element)

    if e.elements[element]['by'] == 'title':
        assert data['text'] != g.driver.title


def input(step):
    data = step['data']
    element = step['element']
    element_location = locating_element(element)

    if step['data'].get('清除文本', '') == '否' or step['data'].get('clear', '').lower() == 'no':
        pass
    else:
        element_location.clear()

    for key in data:
        if key.startswith('text'):
            if isinstance(data[key], tuple):
                element_location.send_keys(*data[key])
            elif element_location:
                element_location.send_keys(data[key])
            sleep(0.5)
        if key == 'word':  #逐字输入
            for d in data[key]:
                element_location.send_keys(d)
                sleep(0.3)
    return element_location


def click(step):
    element = step['element']
    data = step['data']
    if isinstance(element, str):
        element_location = locating_element(element, 'CLICK')
        if element_location:
            try:
                element_location.click()
            except ElementClickInterceptedException:  # 如果元素为不可点击状态，则等待1秒，再重试一次
                sleep(1)
                if data.get('mode'):
                    g.driver.execute_script("arguments[0].click();", element_location)
                else:
                    element_location.click()
    elif isinstance(element, list):
        for _e in element:
            element_location = locating_element(_e, 'CLICK')
            try:
                element_location.click()
            except ElementClickInterceptedException:  # 如果元素为不可点击状态，则等待1秒，再重试一次
                sleep(1)
                if data.get('mode'):
                    g.driver.execute_script("arguments[0].click();", element_location)
                else:
                    element_location.click()        
            sleep(0.5)
    sleep(0.5)

    # 获取元素其他属性
    output = step['output']
    for key in output:
        if output[key] == 'text':
            g.var[key] = element_location.text
        elif output[key] in ('text…', 'text...'):
            if element_location.text.endswith('...'):
                g.var[key] = element_location.text[:-3]
            else:
                g.var[key] = element_location.text
        else:
            g.var[key] = element_location.get_attribute(output[key])

    # 判断是否打开了新的窗口，并将新窗口添加到所有窗口列表里
    all_handles = g.driver.window_handles
    for handle in all_handles:
        if handle not in w.windows.values():
            w.register(step, handle)

    return element_location


def select(step):
    pass


def hover(step):
    actions = ActionChains(g.driver)
    element = step['element']
    element_location = locating_element(element)
    actions.move_to_element(element_location)
    actions.perform()
    sleep(0.5)

    return element_location


def context_click(step):
    actions = ActionChains(g.driver)
    element = step['element']
    element_location = locating_element(element)
    actions.context_click(element_location)
    actions.perform()
    sleep(0.5)

    return element_location


def double_click(step):
    actions = ActionChains(g.driver)
    element = step['element']
    element_location = locating_element(element)
    actions.double_click(element_location)
    actions.perform()
    sleep(0.5)

    return element_location


def drag_and_drop(step):
    actions = ActionChains(g.driver)
    element = step['element']
    source = locating_element(element[0])
    target = locating_element(element[1])
    actions.drag_and_drop(source, target)
    actions.perform()
    sleep(0.5)


def swipe(step):
    actions = ActionChains(g.driver)
    element = step['element']
    data = step['data']

    source = locating_element(element)
    x = data.get('x', 0)
    y = data.get('y', 0)
    actions.drag_and_drop_by_offset(source, x, y)
    actions.perform()
    sleep(0.5)


def script(step):
    element = step['element']
    value = e.get(element)[1]
    g.driver.execute_script(value)


def message(step):
    data = step['data']
    text = data.get('text', '')
    element = step['element']
    value = e.get(element)[1]

    if value.lower() in ('确认', 'accept'):
        g.driver.switch_to_alert().accept()
    elif value.lower() in ('取消', '关闭', 'cancel', 'close'):
        g.driver.switch_to_alert().dismiss()
    elif value.lower() in ('输入', 'input'):
        g.driver.switch_to_alert().send_keys(text)
        g.driver.switch_to_alert().accept()
    logger.info('--- Switch Frame: Alert')
    w.frame = 'Alert'


def upload(step):
    import win32com.client

    data = step['data']
    element = step['element']
    element_location = locating_element(element)
    file_path = data.get('text', '') or data.get('file', '')

    element_location.click()
    sleep(3)
    shell = win32com.client.Dispatch("WScript.Shell")
    shell.Sendkeys(file_path)
    sleep(2)
    shell.Sendkeys("{ENTER}")
    sleep(2)


def navigate(step):
    element = step['element']

    if element.lower() in ('刷新', 'refresh'):
        g.driver.refresh()
    elif element.lower() in ('前进', 'forward'):
        g.driver.forward()
    elif element.lower() in ('后退', 'back'):
        g.driver.back()


def scroll(step):
    data = step['data']
    x = data.get('x')
    y = data.get('y') or data.get('text')

    element = step['element']
    if element == '':
        # if x is None:
        #     x = '0'
        # g.driver.execute_script(
        #     f"window.scrollTo({x},{y})")
        if y:
            g.driver.execute_script(
                f"document.documentElement.scrollTop={y}")
        if x:
            g.driver.execute_script(
                f"document.documentElement.scrollLeft={x}")         
    else:
        element_location = locating_element(element)

        if y:
            g.driver.execute_script(
                f"arguments[0].scrollTop={y}", element_location)
        if x:
            g.driver.execute_script(
                f"arguments[0].scrollLeft={x}", element_location)               


from super_sweetest.log import logger
from super_sweetest.config import all_keywords, comma_lower


def escape(data):
    # 先把转义字符替换掉
    # return data.replace('\\,', comma_lower).replace('\\，', comma_upper).replace('\\=', equals)
    return data.replace('\\,', comma_lower)


def recover(data):
    # 再把转义字符恢复
    # return data.replace(comma_lower, ',').replace(comma_upper, '，').replace(equals, '=')
    return data.replace(comma_lower, ',')


def check_keyword(kw):
    try:
        keyword = all_keywords.get(kw)
        return keyword
    except:
        logger.exception('Keyword:%s is not exist' % kw)
        exit()


def data_format(data):
    data = escape(data)
    if ',,' in data:
        data_list = data.split(',,')
    else:
        # data = data.replace('，', ',')  # 中文逗号不再视为分隔符
        data_list = []
        if data:
            data_list = data.split(',')
    data_dict = {}
    for data in data_list:
        # 只需要分割第一个'='号
        d = data.split('=', 1)
        d[-1] = recover(d[-1])  # 只有值需要转义恢复，<元素属性> or <变量名> 不应该出现转义字符
        if len(d) == 1:
            # 如果没有=号分割，说明只有内容，默认赋值给 text
            if not data_dict.get('text'):
                data_dict['text'] = d[0]
        elif len(d) == 2:
            d[0] = d[0].strip()  # 清除 <元素属性> 2边的空格，如果有的话
            data_dict[d[0]] = d[1]
        else:
            raise Exception(
                'Error: Testcase\'s Data is error, more "=" or less ","')
    return data_dict


def parse(testsuit):
    '''
    将测试用例解析为可执行参数，如:
    打开首页，解析为：OPEN 127.0.0.1
    '''
    for testcase in testsuit:
        for step in testcase['steps']:
            step['keyword'] = check_keyword(step['keyword'])
            # step['page'], step['custom'], step['element'] = elements_format(
            #     step['page'], step['element'])
            step['data'] = data_format(str(step['data']))
            step['expected'] = data_format(str(step['expected']))
            step['output'] = data_format(step['output'])



# coding:utf-8

# @Time: 2020/3/5 10:57
# @Auther: liyubin

import json

"""
公共读写方法 / 配置读写
"""


def read_lib_app_config(app_config_file='./lib/app_config.json'):
    """
    # 读取./lib/app_config.json配置文件
    :param app_config_file: json配置路径
    :return:
    """
    try:
        with open(app_config_file, 'r', encoding='utf-8') as fp:
            app_config = eval(fp.read())
    except:
        # 未配置json文件时读取默认或在运行程序中配置得dict
        DESIRED_CAPS_DEMO = {"DESIRED_CAPS": {
                                                "platformName": "Air_Android",
                                                "deviceName": "23fdgdg",
                                                "platformVersion": "9",
                                                "appPackage": "com.app.packagename",
                                                "connect_type_list": ["", "javacap"],
                                                "stop_app": "True",
                                                "appActivity": "com.name.name.ui.MainActivity",
                                                "noReset": "True",
                                                "chromeOptions": {
                                                    "androidProcess": "com.tencent.mm:appbrand0"}
                                            },
                                                "SERVER_URL": "http://127.0.0.1:4723/wd/hub"
                                            }

        print('未获取到 ./lib/app_config.json 配置文件或内容，请在json文件配置或在运行程序中配置如下信息：\n{}'.format(DESIRED_CAPS_DEMO))
        return None, None
    # 环境配置信息
    return app_config.get('DESIRED_CAPS'), app_config.get('SERVER_URL')


def sync_connect(logger):
    """
     同步connect_type信息到平台
    :return:
    """
    import requests
    host = 'http://atp.leedarson.com' # 需配置自动化平台IP
    connect_url = host + '/software/sweetest/mobile/collect/'
    connect_type_ = read_data('./connect_type.json', flag='', mode='r')
    data_ = {'connect_type': connect_type_}
    res = requests.post(connect_url, data_).json()
    logger.info('--- %s ---'%res)


def write_data(file_name, data_, flag='', mode='w+'):
    if flag.lower() == 'json' and mode.lower() == 'w+':
        with open(file_name, 'w+', encoding='utf-8') as fp:
            json.dump(data_, fp)
    elif flag.lower() == 'eval' and mode.lower() == 'w+':
        with open(file_name, 'w+', encoding='utf-8')as fp:
            fp.write(eval(data_))
    elif mode.lower() == 'w+':
        with open(file_name, 'w+', encoding='utf-8')as fp:
            fp.write(data_)
    else:
        with open(file_name, 'a+', encoding='utf-8')as fp:
            fp.write(data_)
    return True


def read_data(file_name, flag='json', mode='r'):
    if mode == 'r':
        with open(file_name, 'r', encoding='utf-8')as fp:
            data_ = fp.read()
    elif mode == 'rb':
        with open(file_name, 'rb')as fp: # 不加encoding
            data_ = fp.read()
    else:
        with open(file_name, 'r+', encoding='utf-8')as fp:
            data_ = fp.read()
    if flag == 'json':
        try:
            return json.loads(data_)
        except Exception as e:
            raise 'Json文件内容格式错误' + str(e)
    elif flag == 'eval':
        return eval(data_)
    else:
        return data_



# coding=utf-8

# @Time: 2020/4/14 11:00
# @Auther: liyubin

import re
import os
import time
import requests
from super_sweetest.config import BASE_DIR

"""
使用xcodebuild,启动WebDriverAgent托管测试服务
devicename 不建议使用名称：iphone，否则将kill所有iphone名称服务
首次启动需xcode安装 WebDriverAgent app，并授权描述文件
"""


def run_iproxy8100():
    """
    启动条件 必须在ios_tagent 前启动，否则出现不可估计的错误
    使用iproxy做端口转发, 使用http://127.0.0.1:8100连接airtest
    http://127.0.0.1:8100/status      json串
    http://127.0.0.1:8100/inspector   手机屏幕的投影
    :return:
    """
    iproxy_code = 'nohup iproxy 8100 8100 &'
    os.popen(iproxy_code)
    time.sleep(3)


def post_iproxy_status():
    """
    发送请求到iproxy http://127.0.0.1:8100/status 判断是否连接成功的标示
    :return:
    """
    try:
        res = requests.get(url='http://127.0.0.1:8100/status').json()
        state = res.get('value').get('state') if res else '未获取到返回值'
        state = 'success' if state == 'success' else 'fail'
        return state
    except:
        return '未获取到返回值'


def localtime():
    """获取当前时间"""
    localtime_ = time.asctime(time.localtime(time.time()))
    return localtime_


def get_ios_devices():
    """
    get ios devices
    :return: devices_list  [str,]
    :return: device_desc_list   [dict,dict1]
    """
    ins_devices = 'Instruments -s devices'
    ios_device_text = list(os.popen(ins_devices).readlines())

    device_list = []
    device_desc_list = []
    for devi in ios_device_text:
        device_ = devi.strip('\n')
        if 'Known Devices:' not in device_ and 'Mac' not in device_ and 'Simulator' not in device_:
            version = re.findall(r'[(](.*?)[)]', device_)[0]
            device = device_.split(' (')[0]
            assert device.lower() != 'iphone', '名称不能为：iphone， 请重新设置'
            device_list.append(device)
            device_desc_list.append({'device_name': device, 'version': version})
    print(localtime() + ' --- device list online: {} --- '.format(device_list))
    return device_list, device_desc_list


def logs_path():
    """项目路径 join build_logs"""
    path_ = os.path.join(BASE_DIR, 'build_logs')
    os.mkdir(path_) if not os.path.exists(path_) else path_
    return path_


def get_server_url(devicename):
    """
    获取ios_tagent_server输出信息中ServerURLHere
    :return: server_url
    :用法：connect_device("ios:///" + server_url)
    """

    print("----------- %s Get Server URL ----------------" % devicename)
    build_log = os.path.join(logs_path(), '{}_build_log.txt'.format(devicename))
    with open(build_log, 'rb')as fp:
        build_log = str(fp.read())

    serverurl = re.findall(r"http://(.+?)<-ServerURLHere", build_log)
    if serverurl:
        print("----------- %s ServerURLHere is %s ----------------" % (devicename, serverurl[0]))
        url = serverurl[0]
    else:
        print("----------- %s ServerURLHere is no ----------------" % devicename)
        url = 'retry server'
        time.sleep(8)
    return url


def run_ios_tagent_server(iostagen_file, platform, devicename):
    """
    非阻塞方式脱离终端在后台运行托管测试服务："nohup "+build_code
    通过xcodebuild 托管指定的手机的iOS-Tagent测试服务

    :-derivedDataPath：产生的缓存文件放在./output目录下
    :  configuration：编译环境，选择Debug/Release
    :-destination :选择test时的目标设备和系统版本号
    """

    assert devicename.lower() != 'iphone', '名称不能为：iphone， 请重新设置'
    build_log = os.path.join(logs_path(), '{}_build_log.txt'.format(devicename))
    # 实时写入日志
    build_code = "xcodebuild test \
                 -project %s/WebDriverAgent.xcodeproj \
                 -scheme WebDriverAgentRunner  \
                 -destination 'platform=%s,name=%s' > $'%s'" % (
        iostagen_file, platform, devicename, build_log)

    print("----------- %s WebDriverAgent Server Start ----------------" % devicename)
    # 非阻塞方式脱离终端在后台运行托管测试服务
    os.popen("nohup %s &" % build_code)
    time.sleep(15)


def kill_ios_tagent_server(devicename):
    """
    kill devicename 对应的WebDriverAgentRunner服务进程
    非阻塞
    """

    assert devicename.lower() != 'iphone', '名称不能为：iphone， 请重新设置'
    print("----------- %s WebDriverAgent Server Stop ----------------" % devicename)
    kill_code = "ps -ef | grep '%s' | grep -v grep | awk '{print $2}' | xargs kill -9" % devicename
    os.popen("nohup %s &" % kill_code)

    # kill iproxy
    kill_iproxy_code = "ps -ef | grep '%s' | grep -v grep | awk '{print $2}' | xargs kill -9" % 'iproxy'
    os.popen("nohup %s &"%kill_iproxy_code)

    build_log = os.path.join(logs_path(), '{}_build_log.txt'.format(devicename))
    if os.path.exists(build_log):
        os.remove(build_log)
    time.sleep(10)


if __name__ == '__main__':

    iostagen_file_ = '/Users/liyubin1/Desktop/iOS-Tagent11'
    # platform_ = 'iOS Simulator'  # 模拟器
    # deviceName_ = 'iPhone 11'


    platform_ = 'iOS'
    deviceName_ = 'LDS-Z-C5197'
    run = 2

    if run == 1:
        for i in range(3):
            print(i)
            run_iproxy8100()
            run_ios_tagent_server(iostagen_file_, platform_, deviceName_)
            # print(get_server_url(deviceName_))

            print('开始手动连接手机')
            status = post_iproxy_status()
            print('status: ', status, i)
            time.sleep(15)
    elif run == 2:
        kill_ios_tagent_server(deviceName_)

    else:
        status = post_iproxy_status()
        print('status: ', status)


import os
import sys
import shutil
import zipfile
from pathlib import Path
from super_sweetest.autotest import Autotest
from super_sweetest.report import reporter


def extract(zfile, path):
    f = zipfile.ZipFile(zfile, 'r')
    for file in f.namelist():
        f.extract(file, path)


def sweetest():
    """命令行直接改py为zip解压demo"""
    sweetest_dir = Path(__file__).resolve().parents[0]
    example_dir_ = sweetest_dir / 'example' / 'sweetest_example.py'
    if os.path.exists(example_dir_):
        new_example_dir = sweetest_dir / 'example' / 'sweetest_example.zip'
        os.rename(example_dir_, new_example_dir)
    example_dir = sweetest_dir / 'example' / 'sweetest_example.zip'
    extract(str(example_dir), Path.cwd())

    print('\n文档: https://sweeter.io\n公众号：喜文测试\nsuper_sweetest自动化Q Q 群：630589174 (验证码：python)注意首字母小写')
    print('\n\n生成 super_sweetest example 成功\n快速体验，请输入如下命令(进入示例目录，启动运行脚本):\n\ncd sweetest_example\npython start.py')


def report():
    sweetest_dir = Path(__file__).resolve().parents[0]
    report_dir = sweetest_dir / 'example' / 'report.zip'
    extract(str(report_dir), Path.cwd())


# coding:utf-8

# @Time: 2020/3/5 10:57
# @Auther: liyubin


import json
import time
import requests
from super_sweetest.client.app.android_ios_cli import create_android_ios_work
from super_sweetest.servers.common import write_data
from super_sweetest.servers.ios_server import get_ios_devices
from super_sweetest.servers.cmd_server import get_devices, get_mobile_desc

"""
1、 手机信息同步，单独运行保障执行用例过程中同步

2、 检测接入手机和创建Android/ios自动化work 目录

    部署需关注：配置 TEST_TYPE 、PLATFORM_URL
"""



def sync_devices(platform_url, test_type):
    """
    同步mobile状态/手机详情
    """
    localtime = time.asctime(time.localtime(time.time()))

    if test_type == 1:
        device_list = get_devices()  # 获取设备列表
        mobile_desc_list = get_mobile_desc(device_list)  # 获取处理后手机详情
    else:
        device_list, mobile_desc_list = get_ios_devices()  # 获取ios设备列表

    write_data('./device_list.json', device_list, 'json', 'w+')  # 写入连接设备

    try:
        # 同步手机详情信息
        connect_url = platform_url + 'software/sweetest/mobile/collect/'

        res = requests.post(connect_url, {'mobile_desc_list': json.dumps(mobile_desc_list)}).json()

        if res['code'] == 200:
            print(localtime + ' --- sync device success --- ')
        else:
            print(localtime + ' --- sync device fails --- ')
    except Exception as e:
        pass
    return mobile_desc_list


def run_sync_device_(test_type=None, platform_url=None, copy_file_list=None, iosTange_file=None, diy_redis_ip=None):
    """
    主方法
    :return:
    """
    TEST_TYPE = 1  if not test_type else test_type  # 创建 Android 标识 1, iOS 标识 2

    PLATFORM_URL = 'http://atp.leedarson.com/'  if not platform_url else platform_url  # 正式
    # PLATFORM_URL = 'http://127.0.0.1:8000/'  # 本地

    COPY_FILE_LIST = ['test_case.py', 'run_this.py', '__init__.py']   if not copy_file_list else copy_file_list  # copy_file 下需要拷贝的文件

    IOSTANGE_FILE = '/Users/liyubin1/Desktop/xcode11.5--iOS-Tagent'  if not iosTange_file else iosTange_file  # iOS-Tagent 服务在mac的目录位置

    DIY_REDIS_IP = '172.24.185.62' if not diy_redis_ip else diy_redis_ip  # 自定义配置不同的redis地址

    while True:
        time.sleep(5)
        try:
            # 同步手机
            MOBILE_LIST = sync_devices(PLATFORM_URL, TEST_TYPE)
            # 检测接入手机和创建Android/ios自动化work 目录
            create_android_ios_work(MOBILE_LIST, TEST_TYPE, PLATFORM_URL, COPY_FILE_LIST, IOSTANGE_FILE, DIY_REDIS_IP)
        except Exception as e:
            print('同步手机失败：', e)



from super_sweetest.globals import g
from super_sweetest.log import logger


class Windows:
    def __init__(self):
        self.init()

    def init(self):
        # 当前窗口名字，如：'新版门户首页窗口', 'HOME'
        self.current_window = ''
        # 所有窗口名字--窗口handle映射表，如:
        # {'新版门户首页窗口': 'CDwindow-3a12c86f-1986-4c02-ba7b-5a0ed94c5963', 'HOME': 'CDwindow-a3f0c44c-d269-4ff0-af38-c31ad70c69e3'}
        self.windows = {}
        # 当前frame名字
        self.frame = 0
        # 所有页面--窗口名字映射表，如：{'门户首页': '新版门户首页窗口'}
        self.pages = {}
        # 新开窗口标志
        self.new_window_flag = True
        # App context
        self.current_context = 'NATIVE_APP'

    def switch_window(self, page):
        if self.new_window_flag:
            if page in list(self.pages):
                page = '通用'
                g.current_page = '通用'
            self.new_window_flag = False

        if page != '通用':
            if page not in list(self.pages):
                # 如果当前页未注册，则需要先清除和当前窗口绑定的页面
                for k in list(self.pages):
                    if self.current_window == self.pages[k]:
                        self.pages.pop(k)
                # 在把当前窗口捆定到当前页面
                self.pages[page] = self.current_window

            elif self.pages[page] != self.current_window:
                # 如果当前窗口为 HOME，则关闭之
                if self.current_window == 'HOME':
                    g.driver.close()
                    self.windows.pop('HOME')
                # 再切换到需要操作的窗口
                tw = self.windows[self.pages[page]]
                logger.info('--- Switch Windows: %s' % repr(tw))
                g.driver.switch_to_window(tw)
                self.current_window = self.pages[page]
                logger.info('--- Current Windows: %s' %
                            repr(self.current_window))

    def switch_frame(self, frame):
        if frame.strip():
            frame = [x.strip() for x in frame.split('|')]
            if frame != self.frame:
                if self.frame != 0:
                    g.driver.switch_to.default_content()
                for f in frame:
                    logger.info('--- Frame Value:  %s' % repr(f))
                    if f.startswith('#'):
                        f = int(f[1:])
                    elif '#' in f:
                        from super_sweetest.testcase import elements_format
                        from super_sweetest.locator import locating_element
                        element = elements_format('通用', f)[2]
                        f = locating_element(element)
                    logger.info('--- Switch Frame: %s' % repr(f))
                    g.driver.switch_to.frame(f)
                self.frame = frame
        else:
            if self.frame != 0:
                g.driver.switch_to.default_content()
                self.frame = 0

    def open(self, step):
        # 查看当前窗口是否已经注册到 windows 映射表
        c = self.windows.get(self.current_window, '')
        # 如果已经存在，则需要清除和当前窗口绑定的页面
        if c:
            for k in list(self.pages):
                if self.current_window == self.pages[k]:
                    self.pages.pop(k)
            # 并从映射表里剔除
            self.windows.pop(self.current_window)

        # 获取当前窗口handle
        handle = g.driver.current_window_handle
        # 注册窗口名称和handle
        self.register(step, handle)

    def register(self, step, handle):
        # 如果有提供新窗口名字，则使用该名字，否则使用默认名字：HOME
        #new_window = step['data'].get('新窗口', 'HOME')
        # 新窗口 变为 标签页名，兼容原有格式
        new_window = 'HOME'
        for k in ('新窗口', '标签页名', 'tabname'):
            if step['data'].get(k):
                new_window = step['data'].get(k)
        # 已存在同名的窗口，则
        if new_window in self.windows:
            # 1. 清除和当前窗口同名的旧窗口绑定的页面
            for k in list(self.pages):
                if new_window == self.pages[k]:
                    self.pages.pop(k)

            # 2. 切换到同名旧窗口去关闭它
            g.driver.switch_to_window(self.windows[new_window])
            g.driver.close()
            # 3. 并从窗口资源池 g.windows 里剔除
            self.windows.pop(new_window)
        # 然后切回当前窗口
        g.driver.switch_to_window(handle)
        # 再添加到窗口资源池 g.windows
        self.windows[new_window] = handle
        # 把当前窗口名字改为新窗口名称
        self.current_window = new_window
        # 新窗口标志置为是
        self.new_window_flag = True

    def close(self):
        all_handles = g.driver.window_handles
        for handle in all_handles:
            # 切换到每一个窗口,并关闭它
            g.driver.switch_to_window(handle)
            g.driver.close()
            logger.info('--- Close th Windows: %s' % repr(handle))

    def switch_context(self, context):
        if context.strip() == '':
            context = 'NATIVE_APP'
        # logger.info('--- ALL   Contexts:%s' % g.driver.contexts)
        # logger.info('--- Input  Context:%s' % repr(context))
        if context != self.current_context:
            if context == '':
                context = None
            logger.info('--- Switch Context:%s' % repr(context))
            g.driver.switch_to.context(context)
            self.current_context = context


w = Windows()



import os
from super_sweetest.log import logger


def copy(step):
    cwd = os.getcwd()
    source = step['element']
    destination = step['data']['text']

    if step['page']: 
        os.chdir(step['page'])

    code = 0
    if os.name == 'nt':
        code = os.system(f'COPY /Y {source} {destination}')
    if os.name == 'posix':
        code = os.system(f'cp -f -R {source} {destination}')    

    if step['page']: 
        os.chdir(cwd)

    if code != 0:
        raise Exception(f'COPY {source} {destination} is failure, code: {code}')


def move(step):
    cwd = os.getcwd()
    source = step['element']
    destination = step['data']['text']

    if step['page']: 
        os.chdir(step['page'])

    code = 0
    if os.name == 'nt':
        code = os.system(f'MOVE /Y {source} {destination}')
    if os.name == 'posix':
        code = os.system(f'mv -f {source} {destination}')    

    if step['page']: 
        os.chdir(cwd)

    if code != 0:
        raise Exception(f'MOVE {source} {destination} is failure, code: {code}')


def remove(step):
    cwd = os.getcwd()
    path = step['element']

    if step['page']: 
        os.chdir(step['page'])

    code = 0
    if os.name == 'nt':
        code = os.system(f'del /S /Q {path}')
    if os.name == 'posix':
        code = os.system(f'rm -f {path}')    

    if step['page']: 
        os.chdir(cwd)

    if code != 0:
        raise Exception(f'REMOVE {path} is failure, code: {code}')


def rmdir(step):
    cwd = os.getcwd()
    path = step['element']

    if step['page']: 
        os.chdir(step['page'])

    code = 0
    if os.name == 'nt':
        code = os.system(f'rd /S /Q {path}')
    if os.name == 'posix':
        code = os.system(f'rm -rf {path}')    

    if step['page']: 
        os.chdir(cwd)

    if code != 0:
        raise Exception(f'RERMDIR {path} is failure, code: {code}')


def mkdir(step):
    cwd = os.getcwd()
    path = step['element']

    if step['page']: 
        os.chdir(step['page'])

    code = 0
    if os.name == 'nt':
        code = os.system(f'mkdir {path}')
    if os.name == 'posix':
        code = os.system(f'mkdir -p {path}')    

    if step['page']: 
        os.chdir(cwd)

    if code != 0:
        raise Exception(f'MKDIR {path} is failure, code: {code}')


def exists(step):
    cwd = os.getcwd()
    path = step['element']

    if step['page']: 
        os.chdir(step['page'])

    result = os.path.exists(path)

    if step['page']: 
        os.chdir(cwd)

    if not result:
        raise Exception(f'{path} is not exists')    


def not_exists(step):
    try:
        exists(step)
    except:
        pass
    else:
        path = step['element']
        raise Exception(f'{path} is a exists')
    

def is_file(step):
    cwd = os.getcwd()
    path = step['element']

    if step['page']: 
        os.chdir(step['page'])

    result = os.path.isfile(path)

    if step['page']: 
        os.chdir(cwd)

    if not result:
        raise Exception(f'{path} is not file')


def not_file(step):
    try:
        is_file(step)
    except:
        pass
    else:
        path = step['element']
        raise Exception(f'{path} is a file')

        
def is_dir(step):
    cwd = os.getcwd()
    path = step['element']

    if step['page']: 
        os.chdir(step['page'])

    result = os.path.isdir(path)

    if step['page']: 
        os.chdir(cwd)

    if not result:
        raise Exception(f'{path} is not dir')


def not_dir(step):
    try:
        is_dir(step)
    except:
        pass
    else:
        path = step['element']
        raise Exception(f'{path} is a dir')


def command(step, name=None):
    cwd = os.getcwd()
    cmd = step['element']

    if name and os.name != name:
        logger.info(f'COMMAND: this OS is not {name}, "{cmd}" is skipped')
        return


    if step['page']: 
        os.chdir(step['page'])

    code = os.system(cmd)  

    if step['page']: 
        os.chdir(cwd)

    if code != 0:
        raise Exception(f'COMMAND: "{cmd}" is failure, code: {code}')


def shell(step):
    command(step, 'posix')


def cmd(step):
    command(step, 'nt')


web_keywords = {
    '打开': 'OPEN',
    'OPEN': 'OPEN',
    '检查': 'CHECK',
    'CHECK': 'CHECK',
    '#检查': 'NOTCHECK',
    '#CHECK': 'NOTCHECK',
    '输入': 'INPUT',
    'INPUT': 'INPUT',
    '点击': 'CLICK',
    'CLICK': 'CLICK',
    '选择': 'SELECT',
    'SELECT': 'SELECT',
    '移动到': 'HOVER',
    '悬停': 'HOVER',
    'HOVER': 'HOVER',
    '右击': 'CONTEXT_CLICK',
    'CONTEXT_CLICK': 'CONTEXT_CLICK',
    '双击': 'DOUBLE_CLICK',
    'DOUBLE_CLICK': 'DOUBLE_CLICK',
    '拖拽': 'DRAG_AND_DROP',
    'DRAG_AND_DROP': 'DRAG_AND_DROP',
    '滑动': 'SWIPE',
    'SWIPE': 'SWIPE',
    '脚本': 'SCRIPT',
    'SCRIPT': 'SCRIPT',
    '对话框': 'MESSAGE',
    'MESSAGE': 'MESSAGE',
    '上传文件': 'UPLOAD',
    'UPLOAD': 'UPLOAD',
    '导航': 'NAVIGATE',
    'NAVIGATE': 'NAVIGATE',
    '滚动条': 'SCROLL',
    'SCROLL': 'SCROLL'
}

common_keywords = {
    '执行': 'EXECUTE',
    'EXECUTE': 'EXECUTE',
    'SQL': 'SQL'
}

http_keywords = {
    'GET': 'GET',
    'POST': 'POST',
    'PUT': 'PUT',
    'PATCH': 'PATCH',
    'DELETE': 'DELETE',
    'OPTIONS': 'OPTIONS'
}

mobile_keywords = {
    '检查': 'CHECK',
    'CHECK': 'CHECK',
    '#检查': 'NOTCHECK',
    '#CHECK': 'NOTCHECK',
    '输入': 'INPUT',
    'INPUT': 'INPUT',
    '填写': 'SET_VALUE',
    'SET_VALUE': 'SET_VALUE',
    '点击': 'CLICK',
    'CLICK': 'CLICK',
    '轻点': 'TAP',
    'TAP': 'TAP',
    '按键码': 'PRESS_KEYCODE',  # Android 特有，常见代码 HOME:3, 菜单键：82，返回键：4
    'PRESS_KEYCODE': 'PRESS_KEYCODE',
    '滑动': 'SWIPE',
    'SWIPE': 'SWIPE',
    '划线': 'LINE',
    'LINE': 'LINE',
    '划线解锁': 'LINE_UNLOCK',
    'LINE_UNLOCK': 'LINE_UNLOCK',
    '摇杆': 'ROCKER',
    'ROCKER': 'ROCKER',
    '滚动': 'SCROLL',  # iOS 专用
    'SCROLL': 'SCROLL',
    '拖拽': 'DRAG_AND_DROP',
    'DRAG_AND_DROP': 'DRAG_AND_DROP',
    '摇晃': 'SHAKE',  # 貌似 Android 上不可用
    'SHAKE': 'SHAKE',
    '快速滑动': 'FLICK',
    'FLICK': 'FLICK',
    '滑动元素': 'FLICK_ELEMENT',
    'FLICK_ELEMENT': 'FLICK_ELEMENT',
    '长按': 'LONG_PRESS',
    'LONG_PRESS': 'LONG_PRESS',
    '缩小': 'PINCH',
    'PINCH': 'PINCH',
    '放大': 'ZOOM',
    'ZOOM': 'ZOOM',
    '隐藏键盘': 'HIDE_KEYBOARD',  # iOS 专用
    'HIDE_KEYBOARD': 'HIDE_KEYBOARD',
    '命名标签页': 'TAB_NAME',
    'TAB_NAME': 'TAB_NAME',
    '重启': 'LAUNCH_APP',
    'LAUNCH_APP': 'LAUNCH_APP',
    '锁屏状态': 'IS_LOCKED',
    'IS_LOCKED': 'IS_LOCKED',
    '锁屏': 'LOCK',
    'LOCK': 'LOCK',
    '解锁': 'UNLOCK',
    'UNLOCK': 'UNLOCK',
}

air_mobile_keywords = {
    '轻点': 'TAP',
    'TAP': 'TAP',
    '轻点坐标':'TAP_POINT',
    'TAP_POINT':'TAP_POINT',
    '点击': 'CLICK',
    'CLICK': 'CLICK',
    '滑动': 'SWIPE',
    'SWIPE': 'SWIPE',
    '滑动图片': 'SWIPE_PHOTO',
    'SWIPE_PHOTO': 'SWIPE_PHOTO',
    '检查': 'CHECK',
    'CHECK': 'CHECK',
    '#检查': 'NOTCHECK',
    'NOTCHECK': 'NOTCHECK',
    '检查文本':'CHECK_TEXT',
    'CHECK_TEXT':'CHECK_TEXT',
    "#检查文本":'NOTCHECK_TEXT',
    'NOTCHECK_TEXT':'NOTCHECK_TEXT',
    '输入': 'INPUT',
    'INPUT': 'INPUT',
    '等待': 'WAIT_',
    'WAIT_': 'WAIT_',
    '返回键': 'BACK',
    'BACK': 'BACK',
    'HOME键': 'HOME',
    'HOME': 'HOME',
    '菜单键': 'MENU',
    'MENU': 'MENU',
    '电源键': 'POWER',
    'POWER': 'POWER',
    '唤醒': 'WAKE_',
    'WAKE_': 'WAKE_',
    'AI识别': 'AI_',
    'AI_': 'AI_'
}

mqtt_keywords = {
    'SEND': 'SEND',
    'LOGIN_MQTT':'LOGIN_MQTT'
}

windows_keywords = {
    '检查': 'CHECK',
    'CHECK': 'CHECK',
    '菜单': 'MENU_SELECT',
    'MENU_SELECT': 'MENU_SELECT',
    '选择': 'SELECT',
    'SELECT': 'SELECT',
    '点击': 'CLICK',
    'CLICK': 'CLICK',
    '双击': 'DOUBLE_CLICK',
    'DOUBLE_CLICK': 'DOUBLE_CLICK',
    '勾选': 'CHECK_OFF',
    'CHECK_OFF': 'CHECK_OFF',
    '输入': 'INPUT',
    'INPUT': 'INPUT',
    '填写': 'SET_TEXT',
    'SET_TEXT': 'SET_TEXT',
    '按键': 'SEND_KEYS',
    'SEND_KEYS': 'SEND_KEYS',
    '窗口': 'WINDOW',
    'WINDOW': 'WINDOW'
}

files_keywords = {
    '复制': 'COPY',
    'COPY': 'COPY',
    '移动': 'MOVE',
    'MOVE': 'MOVE',
    '删除文件': 'REMOVE',
    'REMOVE': 'REMOVE',
    '删除目录': 'RMDIR',
    'RMDIR': 'RMDIR',
    '创建目录': 'MKDIR',
    'MKDIR': 'MKDIR',
    '路径存在': 'EXISTS',
    'EXISTS': 'EXISTS',
    '路径不存在': 'NOT_EXISTS',
    'NOT_EXISTS': 'NOT_EXISTS',
    '是文件': 'IS_FILE',
    'IS_FILE': 'IS_FILE',
    '是目录': 'IS_DIR',
    'IS_DIR': 'IS_DIR',
    '不是文件': 'NOT_FILE',
    'NOT_FILE': 'NOT_FILE',
    '不是目录': 'NOT_DIR',
    'NOT_DIR': 'NOT_DIR',
    '命令行': 'COMMAND',
    'COMMAND': 'COMMAND',
    'SHELL': 'SHELL',
    'CMD': 'CMD'
}

all_keywords = {}
for keywords in (web_keywords, common_keywords, http_keywords, mobile_keywords, air_mobile_keywords, mqtt_keywords, windows_keywords, files_keywords):
    all_keywords = dict(all_keywords, **keywords)

# 文件名后缀
_testcase = 'TestCase'  # '测试用例'
_elements = 'Elements'  # '页面元素表'
_report = 'Report'  # '测试结果'

# 特殊符号的转换别名
comma_lower = '#$%^&'
comma_upper = '&^%$#'
equals = '%^$#&'
vertical = '&$&*^&A@'

# header = ['用例编号', '用例标题', '前置条件', '测试步骤', '操作', '页面', '元素',
# '测试数据', '预期结果', '输出数据', '优先级', '设计者', '自动化标记', '测试结果', '备注']

header = {
    '用例编号': 'id',
    '用例标题': 'title',
    '前置条件': 'condition',
    '测试步骤': 'step',
    '操作': 'keyword',
    '页面': 'page',
    '元素': 'element',
    '测试数据': 'data',
    '预期结果': 'expected',
    '输出数据': 'output',
    '优先级': 'priority',
    '设计者': 'designer',
    '自动化标记': 'flag',
    '步骤结果': 'score',
    '用例结果': 'result',
    '备注': 'remark'
}

element_wait_timeout = 10  # 等待元素出现超时时间，单位：秒
page_flash_timeout = 90  # 页面刷新超时时间，单位：秒
rgb_flag = True


# 路径配置当前目录上层
import os
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# AI识别配置
HOST = 'https://planettest.cn'
URL_LDS_UI = HOST + '/ai/lds_ui_2' # 识别UI异常
URL_BAIDU_AIP = HOST + '/ai/baidu_aip' # 识别UI文本




# coding:utf-8

# @Time: 2020/3/10 16:42
# @Auther: liyubin


from airtest.core.api import *
from poco.drivers.android.uiautomation import AndroidUiautomationPoco
from super_sweetest.servers.common import read_data, write_data

########################################
# from airtest.core.settings import Settings
# 全局伐值

# Settings.THRESHOLD_STRICT = 0.6  # assert_exists语句的默认阈值，一般比THRESHOLD更高一些
# Settings.THRESHOLD = 0.6  # 其他语句的伐值
# Settings.OPDELAY = 1  # 每一步操作后等待1秒再进行下一步操作
# Settings.FIND_TIMEOUT = 10  # 图像查找超时时间
# Settings.CVSTRATEGY = ["surf", "tpl", "brisk"]  # 修改图片识别算法顺序，只要成功匹配任意一个符合设定阙值的结果，程序就会认为识别成功

########################################

"""
airtest android  服务
"""


class AirAndroid():

    def setup_air_android(self, device_name, app_package, desired_caps):
        """
        connect_type_list: list
        connect_device airtest， init poco
        通过ADB连接本地Android设备
        init_device("Android")
        或者使用connect_device函数
        cap_method=JAVACAP&&ori_method=ADBORI&&touch_method=ADBTOUCH 对应 'javacap', 'ADB orientation', 'ADB touch'
        """

        auto_setup(__file__)

        # 通过不同method连接手机
        global CONNECT_TYPE_LIST, desired_caps_g
        desired_caps_g = desired_caps
        CONNECT_TYPE_LIST = desired_caps_g.get('connect_type_list', '未配置')
        assert isinstance(CONNECT_TYPE_LIST, list), "格式错误 正确格式-> 'connect_type_list':['', 'javacap', 'adbori', 'adbtouch'],"
        c_t = [C_T.lower() for C_T in CONNECT_TYPE_LIST]

        if 'javacap' in c_t and 'adbori' in c_t and 'adbtouch' in c_t:
            method = "?cap_method=JAVACAP&&ori_method=ADBORI&&touch_method=ADBTOUCH"
        elif 'adbori' in c_t and 'adbtouch' in c_t:
            method = "?ori_method=ADBORI&&touch_method=ADBTOUCH"
        elif 'javacap' in c_t and 'adbtouch' in c_t:
            method = "?cap_method=JAVACAP&&touch_method=ADBTOUCH"
        elif 'javacap' in c_t and 'adbori' in c_t:
            method = "?cap_method=JAVACAP&&ori_method=ADBORI"
        elif 'javacap' in c_t:
            method = "?cap_method=JAVACAP"
        else:
            method = ''

        connect_device("Android:///" + device_name + method)

        # install("your apk")
        AirAndroid.put_full(device_name)

        # poco实例化,需先连接设备
        poco = AndroidUiautomationPoco(use_airtest_input=True, screenshot_each_action=False)

        sleep(3)
        start_app(app_package)
        sleep(8)
        return poco

    @staticmethod
    def teardown_air_android(g):
        """
        关闭app，停止poco实例
        :param g:
        """
        AirAndroid.put_null(g.deviceName)
        g.poco.stop_running()
        # os.system('adb -s {} usb'.format(g.deviceName))
        if desired_caps_g.get('stop_app', True):
            stop_app(g.appPackage)
        AirAndroid.sync_connect_log(g.deviceName, CONNECT_TYPE_LIST)

    # 无device_name时只适用首次启动执行端时，否则多次执行导致adb命令异常,不支持多设备连接使用
    # 如提示adb版本不同，将airtest的adb.exe替换android sdk
    @staticmethod
    def put_full(device_name):
        """隐藏指定手机状态栏通知栏"""
        os.system('adb -s {} shell settings put global policy_control immersive.full=*'.format(device_name))

    @staticmethod
    def put_null(device_name):
        """恢复指定手机状态栏导航栏"""
        os.system('adb -s {} shell settings put global policy_control null'.format(device_name))

    @staticmethod
    def sync_connect_log(device_name, method):
        """
        更新添加已运行的log
        :param device_name:
        :param method:
        """
        connect_type_file = './connect_type.json'
        # 首次运行不存在先创建再读取
        if not os.path.exists(connect_type_file):
            write_data(connect_type_file, {}, 'json', 'w+')
        # 读取最新配置
        connect_type = read_data(connect_type_file, 'json', 'r+')
        # 更新/新增 配置
        connect_type[device_name] = method
        # 更新写入配置key：value
        write_data(connect_type_file, connect_type, 'json', 'w+')



# coding=utf-8

# @Time: 2020/3/5 10:57
# @Auther: liyubin

"""
elements 辅助定位
"""


def tap_or_input_elements_index(g, element_location, data_=None):
    """
    处理elements 的 index 点击 和 输入
    :param g: 全局变量
    :param element_location: dict 所需elements/value/index
    :param data_: 输入时参数
    mobile中 input /tap 调用
    """
    if element_location.get('elements') == 'class_name' and isinstance(int(element_location.get('index_')), int):
        if data_:
            g.driver.find_elements_by_class_name(element_location.get('value'))[element_location.get('index_')].clear()
            g.driver.find_elements_by_class_name(element_location.get('value'))[
                element_location.get('index_')].send_keys(data_['text'])
        else:
            g.driver.find_elements_by_class_name(element_location.get('value'))[element_location.get('index_')].click()



import datetime


def today():
    now = datetime.datetime.now()
    return now.strftime('%Y%m%d')


days = ['20180422', '20180423', '20180424','20180425', '20180426', '20180427', '20180428']

def td(t=0):
    day = today()
    td = '19700101'

    for i,d in enumerate(days):
        if d >= day:
            td = days[i+t]
            break
    return td

def test_trade_day():
    assert '20180422' == td(-3)
    assert '20180426' == td(1)



from datetime import datetime
from xml.dom.minidom import Document


class JUnit():

    def __init__(self):
        self.testsuites = []

    def create_suite(self, name, hostname="localhost"):
        suite = TestSuite(name, hostname)
        self.testsuites.append(suite)
        return suite

    def finish(self):
        for suite in self.testsuites:
            if suite.open == True:
                suite.finish()

    def write(self, file):
        self.finish()
        doc = Document()
        root = doc.createElement("testsuites")
        doc.appendChild(root)
        for suite in self.testsuites:
            root.appendChild(suite.to_xml(doc))
        file.write(doc.toprettyxml())


class TestSuite():
    def __init__(self, name, hostname):
        self.properties = []
        self.name = name
        self.hostname = hostname
        self.open = False
        self.cases = []
        self.systemout = None
        self.systemerr = None

    def start(self):
        self.open = True
        self.time = datetime.now()
        self.timestamp = datetime.isoformat(self.time)
        return self

    def create_case(self, name, classname=""):
        if self.open:
            case = TestCase(name, classname)
            self.cases.append(case)
            return case
        else:
            raise Exception(
                "This test suite cannot be modified in its current state")

    def append_property(self, name, value):
        self.properties.append([name, value])

    def finish(self, output=None, error=None):
        if self.open == True:
            self.open = False
            # set the number of test cases, error cases, failed cases, and the amount of time taken in seconds.
            td = datetime.now() - self.time
            self.time = float(
                (td.microseconds + (td.seconds + td.days * 24 * 3600) * 10**6)) / 10**6
            self.errors = 0
            self.high_errors = 0
            self.medium_errors = 0
            self.low_errors = 0
            self.failures = 0
            self.high_failures = 0
            self.medium_failures = 0
            self.low_failures = 0
            self.skipped = 0
            self.disabled = 0
            for case in self.cases:
                if case.state == None:
                    case.error(
                        "XmlUnit Finished", "The test was forced to finish since the suite was finished.")
                status = case.state.lower()
                if status == "failure":
                    self.failures += 1
                    if case.priority == 'H':
                        self.high_failures += 1
                    if case.priority == 'M':
                        self.medium_failures += 1
                    if case.priority == 'L':
                        self.low_failures += 1
                elif status == "error":
                    self.errors += 1
                    if case.priority == 'H':
                        self.high_errors += 1
                    if case.priority == 'M':
                        self.medium_errors += 1
                    if case.priority == 'L':
                        self.low_errors += 1
                elif status == "skipped":
                    self.skipped += 1                          
                elif status == "blocked":
                    self.disabled += 1              
                else:
                    pass
            self.tests = len(self.cases)
            self.output = output
            self.error = error
            return None
        else:
            raise Exception("This test suite is already finished.")

    def to_xml(self, doc):
        node = doc.createElement("testsuite")
        node.setAttribute("name", self.name)
        node.setAttribute("hostname", self.hostname)
        node.setAttribute("timestamp", self.timestamp)
        node.setAttribute("tests", "%s" % self.tests)
        node.setAttribute("failures", "%s" % self.failures)
        node.setAttribute("failures_detail", "H:%s M:%s L:%s" % (
            self.high_failures, self.medium_failures, self.low_failures))
        node.setAttribute("errors", "%s" % (self.errors + self.skipped + self.disabled))
        node.setAttribute("errors_detail", "H:%s M:%s L:%s" % (
            self.high_errors, self.medium_errors, self.low_errors))
        node.setAttribute("time", "%s" % self.time)
        node.setAttribute("skipped", "%s" % self.skipped)         
        node.setAttribute("disabled", "%s" % self.disabled)        
        for case in self.cases:
            node.appendChild(case.to_xml(doc))

        return node


class TestCase():

    def __init__(self, name, classname):
        self.state = None
        self.name = name
        self.classname = classname
        self.priority = 'M'
        return None

    def start(self):
        self.time = datetime.now()
        return self

    def custom(self, state, type, message):
        if self.state != None:
            raise Exception("This test case is already finished.")
        self.state = state
        self.message = message
        self.type = type
        td = datetime.now() - self.time
        self.time = float(
            (td.microseconds + (td.seconds + td.days * 24 * 3600) * 10**6)) / 10**6

    def fail(self, type, message):
        self.custom("failure", type, message)

    def skip(self, type, message):
        self.custom("skipped", type, message)

    def error(self, type, message):
        self.custom("error", type, message)

    def block(self, type, message):
        self.custom("blocked", type, message)

    def succeed(self):
        if self.state != None:
            raise Exception("This test case is already finished.")
        self.state = "success"
        td = datetime.now() - self.time
        self.time = float(
            (td.microseconds + (td.seconds + td.days * 24 * 3600) * 10**6)) / 10**6

    def to_xml(self, doc):
        node = doc.createElement("testcase")
        node.setAttribute("name", self.name)
        node.setAttribute("classname", self.classname)
        node.setAttribute("priority", self.priority)
        node.setAttribute("time", "%s" % self.time)
        if self.state != "success":
            subnode = doc.createElement(self.state)
            subnode.setAttribute("type", self.type)
            subnode.setAttribute("message", self.message)
            node.appendChild(subnode)
        return node



import time
from copy import deepcopy
from super_sweetest.globals import g, timestamp
from super_sweetest.windows import w
from super_sweetest.testcase import TestCase
from super_sweetest.log import logger
from super_sweetest.servers.air_android import AirAndroid
from super_sweetest.servers.air_ios import AirIos
from super_sweetest.servers.mqtt_client import Mqtt_Init
from super_sweetest.servers.common import sync_connect


class TestSuite:
    def __init__(self, testsuite, sheet_name, report, conditions={}):
        self.testsuite = testsuite
        self.sheet_name = sheet_name
        self.report = report
        self.conditions = conditions
        self.result = {}

        # base 在整个测试套件中首先执行一次
        self.base_testcase = {}
        # setup 在每个测试用例执行之前执行一次
        self.setup_testcase = {}
        for testcase in self.testsuite:
            if testcase['condition'].lower() == 'base':
                self.base_testcase = testcase
            elif testcase['condition'].lower() == 'setup':
                self.setup_testcase = testcase
                testcase['flag'] = 'N'  # setup 用例只在执行其他普通用例之前执行
            elif testcase['condition'].lower() == 'snippet':
                g.snippet[testcase['id']] = testcase
                testcase['flag'] = 'N'

    def testsuite_start(self):
        self.result['no'] = g.no
        g.no += 1
        self.result['testsuite'] = self.sheet_name
        self.result['start_timestamp'] = timestamp()

    def testsuite_end(self):
        self.result['end_timestamp'] = timestamp()
        g.testsuite_data[self.sheet_name] = self.result

    def setup(self, testcase, case):
        logger.info('=' * 50)
        if self.setup_testcase:
            logger.info('*** SETUP testcase ***')
        else:
            logger.info('...No SETUP testcase need to run...')

        def run_setup(testcase):
            if testcase:
                tc = TestCase(testcase)
                tc.run()
                if testcase['result'] == 'success':
                    flag = 'Y'
                else:
                    flag = 'N'
            else:
                flag = 'O'
            return flag

        setup_flag = run_setup(deepcopy(self.setup_testcase))

        if setup_flag == 'N':
            base_flag = run_setup(deepcopy(self.base_testcase))
            if base_flag == 'Y':
                setup_flag = run_setup(deepcopy(self.setup_testcase))
                if setup_flag == 'N':
                    testcase['result'] = 'blocked'
                    case.block('Blocked', 'SETUP is not success')
                    logger.warn('Run the testcase: %s|%s blocked, SETUP is not success' % (
                        testcase['id'], testcase['title']))
                    return False
            elif base_flag == 'O':
                testcase['result'] = 'blocked'
                case.block('Blocked', 'SETUP is not success')
                logger.warn('Run the testcase: %s|%s blocked, SETUP is not success' % (
                    testcase['id'], testcase['title']))
                return False

        return True

    def run(self):

        self.testsuite_start()

        # 当前测试用例
        current = {'result': 'success'}
        # 上一个测试用例
        previous = {}

        # 前置条件执行失败标志，即未执行用例阻塞标志
        blcoked_flag = False

        # 1.执行用例
        for testcase in self.testsuite:
            # 根据筛选条件，把不需要执行的测试用例跳过
            flag = False
            for k, v in self.conditions.items():
                if not isinstance(v, list):
                    v = [v]
                if testcase[k] not in v:
                    testcase['result'] = 'skipped'
                    flag = True
            if flag:
                continue

            # 统计开始时间
            testcase['start_timestamp'] = timestamp()
            # xml 测试报告-测试用例初始化
            if testcase['flag'] != 'N':
                # 如果前置条件失败了，直接设置为阻塞
                if blcoked_flag:
                    testcase['result'] = 'blocked'
                    testcase['end_timestamp'] = timestamp()
                    continue

                case = self.report.create_case(
                    testcase['title'], testcase['id'])
                case.start()
                case.priority = testcase['priority']
                # 用例上下文
                previous = current

                current = testcase
            else:
                testcase['result'] = 'skipped'
                # case.skip('Skip', 'Autotest Flag is N')
                # logger.info('Run the testcase: %s|%s skipped, because the flag=N or the condition=snippet' % (
                #     testcase['id'], testcase['title']))
                # 统计结束时间
                testcase['end_timestamp'] = timestamp()
                continue

            if testcase['condition'].lower() not in ('base', 'setup'):
                if testcase['condition'].lower() == 'sub':
                    if previous['result'] != 'success':
                        testcase['result'] = 'blocked'
                        case.block(
                            'Blocked', 'Main or pre Sub testcase is not success')
                        logger.warn('Run the testcase: %s|%s blocked, Main or pre Sub TestCase is not success' % (
                            testcase['id'], testcase['title']))
                        # 统计结束时间
                        testcase['end_timestamp'] = timestamp()
                        continue
                # 如果前置条件为 skip，则此用例不执行前置条件
                elif testcase['condition'].lower() == 'skip':
                    pass
                else:
                    result = self.setup(testcase, case)
                    # if result == 'N':
                    if not result:
                        # 统计结束时间
                        testcase['end_timestamp'] = timestamp()
                        continue

            try:
                tc = TestCase(testcase)
                logger.info('-' * 50)
                tc.run()

                # 统计结束时间
                testcase['end_timestamp'] = timestamp()

                if testcase['result'] == 'success':
                    case.succeed()
                elif testcase['result'] == 'failure':
                    case.fail('Failure', testcase['report'])
                    if testcase['condition'].lower() == 'base':
                        logger.warn('Run the testcase: %s|%s Failure, BASE is not success. Break the AutoTest' % (
                            testcase['id'], testcase['title']))
                        blcoked_flag = True
                        continue
                    if testcase['condition'].lower() == 'setup':
                        logger.warn('Run the testcase: %s|%s failure, SETUP is not success. Break the AutoTest' % (
                            testcase['id'], testcase['title']))
                        blcoked_flag = True
                        continue
            except Exception as exception:
                case.error('Error', 'Remark:%s |||Exception:%s' %
                           (testcase['remark'], exception))
                logger.exception('Run the testcase: %s|%s failure' %
                                 (testcase['id'], testcase['title']))
                if testcase['condition'].lower() == 'base':
                    logger.warn('Run the testcase: %s|%s error, BASE is not success. Break the AutoTest' % (
                        testcase['id'], testcase['title']))
                    blcoked_flag = True
                    continue
                if testcase['condition'].lower() == 'setup':
                    logger.warn('Run the testcase: %s|%s error, SETUP is not success. Break the AutoTest' % (
                        testcase['id'], testcase['title']))
                    blcoked_flag = True
                    continue

        self.report.finish()

        # 2.清理环境
        try:
            time.sleep(4)
            if g.platform.lower() in ('desktop',):
                # w.close()
                g.driver.quit()
                logger.info('--- Quit the Driver: %s' % g.browserName)
            elif g.platform.lower() in ('android',):
                g.driver.quit()
                logger.info('--- Quit the Driver: %s' % g.platform)
            elif g.platform.lower() in ('air_android'):
                air_android_ = AirAndroid()
                air_android_.teardown_air_android(g)
                logger.info('--- Stop the Server and App: %s  for %s' % (g.appPackage, g.platform))
                sync_connect(logger)
            elif g.platform.lower() in ('air_ios'):
                air_ios_ = AirIos()
                air_ios_.teardown_air_ios(g)
                logger.info('--- Stop the Server and App: %s  for %s' % (g.appPackage, g.platform))
            if g.mqtt_client_:
                # 初始化类传{}即可
                mqtt_init = Mqtt_Init({})
                mqtt_init.teardown_mqttclient(g.mqtt_client_)
                logger.info('--- Stop the MQTT Server')
        except:
            logger.exception('Clear the env is fail')

        self.testsuite_end()
